# Lazygal, a lazy static web gallery generator.
# Copyright (C) 2011-2012 Alexandre Rossi <alexandre.rossi@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


import unittest
import os
import configparser
import json


from . import LazygalTestGen
import lazygal.config
from lazygal.generators import WebalbumDir
from lazygal.sourcetree import Directory


class TestConf(LazygalTestGen):

    def setUp(self):
        super().setUp(album=False)

    def test_perdir_conf_old(self):
        """
        Lazygal shall read old-style configuration files in every source
        directory, the parent directory configuration shall apply to child
        directories.
        """

        os.makedirs(os.path.join(self.source_dir, "gal", "subgal"))

        # src_dir/.lazygal
        config = configparser.RawConfigParser()
        config.add_section("template-vars")
        config.set("template-vars", "foo", "root")
        config.set("template-vars", "root", "root")
        config.add_section("webgal")
        config.set("webgal", "image-size", "normal=800x600")
        with open(os.path.join(self.source_dir, ".lazygal"), "a") as f:
            config.write(f)

        # src_dir/gal/.lazygal
        config = configparser.RawConfigParser()
        config.add_section("template-vars")
        config.set("template-vars", "foo", "gal")
        config.set("template-vars", "gal", "gal")
        with open(os.path.join(self.source_dir, "gal", ".lazygal"), "a") as f:
            config.write(f)

        # src_dir/gal/subgal/.lazygal
        config = configparser.RawConfigParser()
        config.add_section("template-vars")
        config.set("template-vars", "foo", "subgal")
        config.set("template-vars", "subgal", "subgal")
        with open(os.path.join(self.source_dir, "gal", "subgal", ".lazygal"), "a") as f:
            config.write(f)

        config = lazygal.config.LazygalConfig()
        config.set("global", "puburl", "http://example.com/album/")
        self.setup_album(config)

        source_gal = self.setup_subgal("gal", ["gal_img.jpg"])
        source_subgal = self.setup_subgal(
            os.path.join("gal", "subgal"), ["subgal_img.jpg"]
        )
        source_root = Directory(self.source_dir, [source_gal], [], self.album)

        dest_path = self.get_working_path()

        dest_subgal = WebalbumDir(source_subgal, [], self.album, dest_path)
        self.assertEqual(
            dest_subgal.config.get("global", "puburl"), "http://example.com/album/"
        )
        self.assertEqual(dest_subgal.config.get("template-vars", "root"), "root")
        self.assertEqual(dest_subgal.config.get("template-vars", "gal"), "gal")
        self.assertEqual(dest_subgal.config.get("template-vars", "subgal"), "subgal")
        self.assertEqual(dest_subgal.config.get("template-vars", "foo"), "subgal")
        self.assertEqual(
            dest_subgal.config.get("webgal", "image-size"),
            [{"name": "normal", "defs": "800x600", "default": True}],
        )

        dest_gal = WebalbumDir(source_gal, [dest_subgal], self.album, dest_path)
        self.assertEqual(
            dest_gal.config.get("global", "puburl"), "http://example.com/album/"
        )
        self.assertEqual(dest_gal.config.get("template-vars", "root"), "root")
        self.assertEqual(dest_gal.config.get("template-vars", "gal"), "gal")
        self.assertRaises(
            lazygal.config.NoOptionError, dest_gal.config.get, "template-vars", "subgal"
        )
        self.assertEqual(dest_gal.config.get("template-vars", "foo"), "gal")
        self.assertEqual(
            dest_gal.config.get("webgal", "image-size"),
            [{"name": "normal", "defs": "800x600", "default": True}],
        )

        dest_root = WebalbumDir(source_root, [dest_gal], self.album, dest_path)
        self.assertEqual(
            dest_root.config.get("global", "puburl"), "http://example.com/album/"
        )
        self.assertEqual(dest_root.config.get("template-vars", "root"), "root")
        self.assertRaises(
            lazygal.config.NoOptionError, dest_root.config.get, "template-vars", "gal"
        )
        self.assertRaises(
            lazygal.config.NoOptionError,
            dest_root.config.get,
            "template-vars",
            "subgal",
        )
        self.assertEqual(dest_root.config.get("template-vars", "foo"), "root")
        self.assertEqual(
            dest_root.config.get("webgal", "image-size"),
            [{"name": "normal", "defs": "800x600", "default": True}],
        )

    def test_perdir_conf(self):
        """
        Lazygal shall read configuration files in every source directory,
        the parent directory configuration shall apply to child directories.
        """

        os.makedirs(os.path.join(self.source_dir, "gal", "subgal"))

        # src_dir/.lazygal
        with open(os.path.join(self.source_dir, ".lazygal"), "a") as f:
            json.dump(
                {
                    "template-vars": {
                        "foo": "root",
                        "root": "root",
                    },
                    "webgal": {
                        "image-size": [
                            {"name": "normal", "defs": "800x600"},
                        ],
                    },
                },
                f,
            )

        # src_dir/gal/.lazygal
        with open(os.path.join(self.source_dir, "gal", ".lazygal"), "a") as f:
            json.dump(
                {
                    "template-vars": {
                        "foo": "gal",
                        "gal": "gal",
                    },
                },
                f,
            )

        # src_dir/gal/subgal/.lazygal
        with open(os.path.join(self.source_dir, "gal", "subgal", ".lazygal"), "a") as f:
            json.dump(
                {
                    "template-vars": {
                        "foo": "subgal",
                        "subgal": "subgal",
                    },
                },
                f,
            )

        config = lazygal.config.LazygalConfig()
        config.set("global", "puburl", "http://example.com/album/")
        self.setup_album(config)

        source_gal = self.setup_subgal("gal", ["gal_img.jpg"])
        source_subgal = self.setup_subgal(
            os.path.join("gal", "subgal"), ["subgal_img.jpg"]
        )
        source_root = Directory(self.source_dir, [source_gal], [], self.album)

        dest_path = self.get_working_path()

        dest_subgal = WebalbumDir(source_subgal, [], self.album, dest_path)
        self.assertEqual(
            dest_subgal.config.get("global", "puburl"), "http://example.com/album/"
        )
        self.assertEqual(dest_subgal.config.get("template-vars", "root"), "root")
        self.assertEqual(dest_subgal.config.get("template-vars", "gal"), "gal")
        self.assertEqual(dest_subgal.config.get("template-vars", "subgal"), "subgal")
        self.assertEqual(dest_subgal.config.get("template-vars", "foo"), "subgal")
        self.assertEqual(
            dest_subgal.config.get("webgal", "image-size"),
            [{"name": "normal", "defs": "800x600"}],
        )

        dest_gal = WebalbumDir(source_gal, [dest_subgal], self.album, dest_path)
        self.assertEqual(
            dest_gal.config.get("global", "puburl"), "http://example.com/album/"
        )
        self.assertEqual(dest_gal.config.get("template-vars", "root"), "root")
        self.assertEqual(dest_gal.config.get("template-vars", "gal"), "gal")
        self.assertRaises(
            lazygal.config.NoOptionError, dest_gal.config.get, "template-vars", "subgal"
        )
        self.assertEqual(dest_gal.config.get("template-vars", "foo"), "gal")
        self.assertEqual(
            dest_gal.config.get("webgal", "image-size"),
            [{"name": "normal", "defs": "800x600"}],
        )

        dest_root = WebalbumDir(source_root, [dest_gal], self.album, dest_path)
        self.assertEqual(
            dest_root.config.get("global", "puburl"), "http://example.com/album/"
        )
        self.assertEqual(dest_root.config.get("template-vars", "root"), "root")
        self.assertRaises(
            lazygal.config.NoOptionError, dest_root.config.get, "template-vars", "gal"
        )
        self.assertRaises(
            lazygal.config.NoOptionError,
            dest_root.config.get,
            "template-vars",
            "subgal",
        )
        self.assertEqual(dest_root.config.get("template-vars", "foo"), "root")
        self.assertEqual(
            dest_root.config.get("webgal", "image-size"),
            [{"name": "normal", "defs": "800x600"}],
        )

    def test_types(self):
        config = lazygal.config.LazygalConfig()

        # bool
        for true in ("1", "yes", "true", "on"):
            config.set("runtime", "quiet", true)
            self.assertTrue(config.get("runtime", "quiet") is True, true)
        for false in ("0", "no", "false", "off"):
            config.set("runtime", "quiet", false)
            self.assertTrue(config.get("runtime", "quiet") is False, false)

        # int
        config.set("webgal", "thumbs-per-page", "2")
        self.assertEqual(config.get("webgal", "thumbs-per-page"), 2)

        # int or false
        config.set("global", "dir-flattening-depth", "False")
        self.assertTrue(config.get("global", "dir-flattening-depth") is False)
        config.set("global", "dir-flattening-depth", "2")
        self.assertEqual(config.get("global", "dir-flattening-depth"), 2)

        # list
        config.set("webgal", "filter-by-tag", "foo, bar")
        self.assertEqual(config.get("webgal", "filter-by-tag"), ["foo", "bar"])

        # dict
        config.set("webgal", "image-size", "medium=foo, normal=bar")
        self.assertEqual(
            config.get("webgal", "image-size"),
            [
                {"name": "medium", "defs": "foo", "default": True},
                {"name": "normal", "defs": "bar", "default": False},
            ],
        )

        # order
        config.set("webgal", "sort-medias", "dirname:reverse")
        self.assertEqual(
            config.get("webgal", "sort-medias"), {"order": "dirname", "reverse": True}
        )

        # false or string
        config.set("webgal", "original-baseurl", "False")
        self.assertTrue(config.get("webgal", "original-baseurl") is False)
        config.set("webgal", "original-baseurl", "./foo")
        self.assertEqual(config.get("webgal", "original-baseurl"), "./foo")

    def test_syntax(self):
        config = lazygal.config.LazygalConfig()
        configw = lazygal.config.LazygalWebgalConfig()

        # not a valid section
        self.assertRaises(ValueError, config.set, "foo", "bar", "baz")
        self.assertRaises(ValueError, configw.set, "runtime", "quiet", "false")

        # not a valid option
        self.assertRaises(ValueError, config.set, "webgal", "foo", "bar")

        # basic option content parsing error
        self.assertRaises(
            ValueError, config.set, "webgal", "dir-flattening-depth", "foo"
        )
        self.assertRaises(ValueError, config.set, "webgal", "image-size", "crappy")
        self.assertRaises(ValueError, config.set, "runtime", "quiet", "foo")


if __name__ == "__main__":
    unittest.main()


# vim: ts=4 sw=4 expandtab



# Lazygal, a lazy static web gallery generator.
# Copyright (C) 2013 Alexandre Rossi <alexandre.rossi@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


import logging


class ProgressConsoleHandler(logging.StreamHandler):

    last_progress_lengh = 0
    progress_msg = ""

    def clear_last_progress(self):
        self.stream.write("\r" + " " * self.last_progress_lengh + "\r")

    def emit(self, record=None):
        try:
            self.clear_last_progress()
            if record is not None:
                self.stream.write(self.format(record) + "\n")
            self.stream.write(self.progress_msg)
            self.last_progress_lengh = len(self.progress_msg)
            self.flush()
        except (KeyboardInterrupt, SystemExit):
            raise
        except:
            self.handleError(record)

    def update_progress(self, s):
        self.progress_msg = s
        self.emit()

    def close(self):
        self.clear_last_progress()
        self.flush()
        super().close()


# vim: ts=4 sw=4 expandtab



# Lazygal, a lazy static web gallery generator.
# Copyright (C) 2015-2020 Alexandre Rossi <alexandre.rossi@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


import logging
import os
import json
import collections
import datetime

from . import make
from . import tplvars


def json_serializer(obj):
    if isinstance(obj, datetime.datetime):
        return obj.isoformat()
    raise TypeError("Type %s is not JSON serializable", type(obj))


def datetime_hook(json_dict):
    for key, value in json_dict.items():
        if type(value) is str:
            try:
                json_dict[key] = datetime.datetime.fromisoformat(value)
            except ValueError:
                pass  # not a datetime in ISO8601 format
    return json_dict


class JSONWebFile(make.FileMakeObject):

    def __init__(self, webgal):
        super().__init__(os.path.join(webgal.path, self.json_filename))
        self.webgal = webgal

        self.data = None
        try:
            with open(self._path, "r") as json_fp:
                self.data = json.load(json_fp, object_hook=datetime_hook)

            if "version" not in self.data or self.data["version"] != self.version:
                raise ValueError("persistent json data version mismatch")
        except FileNotFoundError:
            self._init_data()
        except ValueError as ve:
            logging.debug(ve)
            os.unlink(self._path)
            self.stamp_delete()
            self._init_data()

    def _init_data(self):
        self.data = collections.OrderedDict()
        self.data["version"] = self.version

    def dump(self):
        with open(self._path, "w") as json_fp:
            if self.webgal.config.get("runtime", "debug"):
                indent = 4
            else:
                indent = None
            json.dump(self.data, json_fp, indent=indent, default=json_serializer)


class PersistentIndex(JSONWebFile):

    json_filename = "index.json"
    version = 1

    def __init__(self, webgal):
        super().__init__(webgal)

        for f in self.webgal.config.files:
            self.add_file_dependency(f)
        self.add_dependency(self.webgal.source_dir)
        for src_media in self.webgal.source_dir.medias:
            self.add_dependency(src_media)
        for subgal in self.webgal.subgals:
            self.add_dependency(subgal.pindex)

    def _init_data(self):
        super()._init_data()

        self.data["config"] = {
            "webgal": dict(self.webgal.config["webgal"]),
        }

        self.data["count"] = {
            "media": 0,
            "image": 0,
            "video": 0,
            "subgal": 0,
        }

        self.data["all_count"] = {
            "media": 0,
            "image": 0,
            "video": 0,
        }

        self.data["medias"] = {}

    def __populate_data(self):
        # reset counts
        for t in ("media", "image", "video", "subgal"):
            self.data["count"][t] = 0

        for media in self.webgal.source_dir.medias:
            for t in ("media", media.type):
                self.data["count"][t] = self.data["count"][t] + 1

        to_delete = []
        for media_filename, media_info in self.data["medias"].items():
            if media_filename not in self.webgal.source_dir.medias_names:
                to_delete.append(media_filename)

        for media_filename in to_delete:
            del self.data["medias"][media_filename]

        self.data["all_count"] = {
            "media": self.data["count"]["media"],
            "image": self.data["count"]["image"],
            "video": self.data["count"]["video"],
        }

        self.data["count"]["subgal"] = len(self.webgal.subgals)
        self.data["subgals"] = []
        for subgal in self.webgal.subgals:
            for count_type in self.data["all_count"]:
                self.data["all_count"][count_type] = (
                    self.data["all_count"][count_type]
                    + subgal.pindex.data["all_count"][count_type]
                )

            self.data["subgals"].append(subgal.source_dir.name)

    def load_media(self, src_media):
        if self.webgal.skip_media(src_media):
            return

        self.data["medias"][src_media.filename] = {}

        for key, value in src_media.md.items():
            if (
                not self.webgal.config.get("webgal", "publish-metadata")
                and key == "metadata"
            ):
                v = {}
            else:
                v = value
            self.data["medias"][src_media.filename][key] = v

        i = self.data["medias"][src_media.filename]
        if "location" in i["metadata"] and not self.webgal.config.get(
            "webgal", "keep-gps"
        ):
            i["metadata"]["location"] = None

    def unpublish_media(self, src_media):
        del self.data["medias"][src_media.filename]
        for t in ("media", src_media.type):
            self.data["count"][t] = self.data["count"][t] - 1

    def webgal_info(self):
        dir_info = {}
        if self.webgal.source_dir.metadata:
            dir_info.update(self.webgal.source_dir.metadata.get())

        if "album_name" not in dir_info:
            dir_info["album_name"] = self.webgal.source_dir.human_name

        return dir_info

    def build(self):
        if not self.webgal.has_media_below():
            return

        logging.info("  DUMPJSON %s", self.json_filename)
        self.__populate_data()

        # Create the webgal directory if it does not exist
        if not os.path.isdir(self.webgal.path) and self.webgal.has_media_below():
            logging.info(
                _("  MKDIR %%WEBALBUMROOT%%/%s"), self.webgal.source_dir.strip_root()
            )
            logging.debug("(%s)", self.webgal.path)
            os.makedirs(self.webgal.path)

        self.dump()

    def get_media_count(self, media_type="media"):
        return self.data["count"][media_type]


class WebAssets(JSONWebFile):

    json_filename = "webassets.json"
    version = 1

    def _init_data(self):
        super()._init_data()
        self.data["dirzip"] = None
        self.dirzip = None

    def build(self):
        if self.webgal.has_media():
            logging.info("  JSONWEBASSETS %s", self.json_filename)

            if self.webgal.dirzip:
                dirzip_size = self.webgal.dirzip.size()
                self.data["dirzip"] = {
                    "filename": self.webgal.dirzip.filename,
                    "size": dirzip_size,
                    "sizestr": tplvars.format_filesize(dirzip_size),
                }

            self.dump()


# vim: ts=4 sw=4 expandtab



# Lazygal, a lazy static web gallery generator.
# Copyright (C) 2007-2012 Alexandre Rossi <alexandre.rossi@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


import os
import posixpath
import sys
import logging

import genshi

from . import make
from . import pathutils
from . import genfile
from . import feeds
from . import tplvars


class WebalbumPage(genfile.WebalbumFile):

    def __init__(self, dir, size_name, base_name):
        self.dir = dir
        self.size_name = size_name

        page_filename = self._add_size_qualifier(base_name + ".html", self.size_name)
        super().__init__(os.path.join(dir.path, page_filename), dir)

        self.page_template = None

    def set_template(self, tpl_ident):
        self.page_template = self.load_tpl(tpl_ident)

    def load_tpl(self, tpl_ident):
        tpl = self.dir.album.theme.tpl_loader.load(tpl_ident)
        self.add_file_dependency(tpl.path)
        for subtpl in tpl.subtemplates():
            self.add_file_dependency(subtpl.path)
        return tpl

    def init_tpl_values(self):
        tpl_values = {}
        tpl_values.update(self.dir.tpl_vars)
        return tpl_values

    def _get_osize_links(self, filename):
        osize_index_links = []
        for osize_name in self.dir.browse_sizes:
            osize_info = {}
            if osize_name == self.size_name:
                # No link if we're on the current page
                osize_info["name"] = osize_name
            else:
                osize_info["name"] = osize_name
                osize_info["link"] = self._add_size_qualifier(
                    filename + ".html", osize_name
                )
                osize_info["link"] = pathutils.url_quote(osize_info["link"])
            osize_index_links.append(osize_info)

        return osize_index_links

    def _add_size_qualifier(self, path, size_name=None):
        if size_name is None:
            size_name = self.size_name

        return self.dir._add_size_qualifier(path, size_name)

    def _do_not_escape(self, value):
        return genshi.core.Markup(value)


class WebalbumBrowsePage(WebalbumPage):

    def __init__(self, dir, size_name, webalbum_media):
        self.webalbum_media = webalbum_media
        self.media = self.webalbum_media.media
        super().__init__(dir, size_name, self.media.name)

        self.add_dependency(self.webalbum_media.resized[size_name])
        if (
            self.webalbum_media.original
            and self.webalbum_media.original not in self.deps
        ):
            self.add_dependency(self.webalbum_media.original)

        # Depends on source directory in case an image was deleted
        self.add_dependency(self.dir.source_dir)

        # Depend on the comment file if it exists.
        if self.webalbum_media.media.comment_file_path is not None:
            self.add_file_dependency(self.webalbum_media.media.comment_file_path)

        self.add_dependency(self.dir.sort_task)

        self.set_template("browse.thtml")
        self.load_tpl(self.media.type + ".thtml")

    def add_extra_vals(self, tpl_values):
        tpl_values.update(tplvars.media_vars(self, self.webalbum_media).full())

    def build(self):
        page_rel_path = self.rel_path(self.dir.flattening_dir)
        logging.info(_("  XHTML %s"), page_rel_path)
        logging.debug("(%s)", self._path)

        tpl_values = self.init_tpl_values()

        # Breadcrumbs
        tpl_values["webgal_path"] = tplvars.Webgal(self, self.dir).path()

        tpl_values["name"] = self.media.filename
        tpl_values["mediatype"] = self.media.type
        tpl_values["dir"] = self.dir.source_dir.strip_root()

        prev = self.webalbum_media.previous
        if prev:
            tpl_values["prev_link"] = tplvars.Media(self, prev).link()

        next = self.webalbum_media.next
        if next:
            tpl_values["next_link"] = tplvars.Media(self, next).link()

        tpl_values["index_link"] = self._add_size_qualifier(
            "index.html", self.size_name
        )
        if self.dir.should_be_flattened():
            index_rel_dir = self.dir.flattening_dir.source_dir.rel_path(
                self.dir.source_dir
            )
            tpl_values["index_link"] = index_rel_dir + tpl_values["index_link"]

        tpl_values["osize_links"] = self._get_osize_links(self.media.name)
        tpl_values["rel_root"] = (
            pathutils.url_path(self.dir.source_dir.rel_root()) + "/"
        )

        if self.dir.feed is not None:
            tpl_values["feed_url"] = os.path.relpath(self.dir.feed.path, self.dir.path)
            tpl_values["feed_url"] = pathutils.url_path(tpl_values["feed_url"])
            tpl_values["feed_url"] = pathutils.url_quote(tpl_values["feed_url"])
        else:
            tpl_values["feed_url"] = None

        self.add_extra_vals(tpl_values)

        self.page_template.dump(tpl_values, self._path)


class WebalbumIndexPage(WebalbumPage):

    FILENAME_BASE_STRING = "index"

    def __init__(self, dir, size_name, page_number, subgals, galleries):
        page_paginated_name = self._get_paginated_name(page_number)
        super().__init__(dir, size_name, page_paginated_name)

        self.page_number = page_number
        self.subgals = subgals
        self.galleries = galleries

        for dir, medias in self.galleries:
            self.add_dependency(dir.source_dir.metadata)
            if dir is not self.dir:
                dir.flattening_dir = self.dir

            self.add_dependency(dir.pindex)
            self.add_dependency(dir.webassets)
            self.add_dependency(dir.sort_task)

            if size_name in dir.browse_sizes:
                for media in medias:
                    if media.thumb:
                        self.add_dependency(media.thumb)
                    if self.dir.album.theme.kind == "static":
                        self.add_dependency(media.browse_pages[size_name])
                        # Ensure dir depends on browse page (usefull for cleanup
                        # checks when dir is flattenend).
                        if self.dir.should_be_flattened():
                            dir.add_dependency(media.browse_pages[size_name])
            else:
                logging.warning(
                    _(
                        "  Size '%s' is not available in '%s' due to configuration: medias won't be shown on index."
                    ),
                    size_name,
                    dir.path,
                )

        if self.dir.album.theme.kind == "static":
            self.set_template("dirindex.thtml")
        elif self.dir.album.theme.kind == "dynamic":
            self.set_template("dynindex.thtml")

    def _get_paginated_name(self, page_number=None):
        if page_number is None:
            page_number = self.page_number
        assert page_number is not None

        if page_number < 1:
            return WebalbumIndexPage.FILENAME_BASE_STRING
        else:
            return "_".join([WebalbumIndexPage.FILENAME_BASE_STRING, str(page_number)])

    def _get_related_index_fn(self):
        return self._add_size_qualifier(
            WebalbumIndexPage.FILENAME_BASE_STRING + ".html", self.size_name
        )

    def _get_onum_links(self):
        onum_index_links = []
        for onum in range(0, self.dir.break_task.how_many_pages()):
            onum_info = {
                "name": onum + 1,
            }
            if onum != self.page_number:  # No link if we're on the current page
                filename = self._get_paginated_name(onum)
                onum_info["link"] = self._add_size_qualifier(
                    filename + ".html", self.size_name
                )
                onum_info["link"] = pathutils.url_quote(onum_info["link"])
            onum_index_links.append(onum_info)

        return onum_index_links

    def _get_subgal_links(self):
        subgal_links = []
        for subgal in self.dir.subgals:
            dir_info = tplvars.Webgal(self, subgal).link_info()
            subgal_links.append(dir_info)
        return subgal_links

    def build(self):
        logging.info(_("  XHTML %s"), os.path.basename(self._path))
        logging.debug("(%s)", self._path)

        values = self.init_tpl_values()

        # Breadcrumbs (current is static, see dirindex.thtml, that's why the
        # last item of the list is removed).
        values["webgal_path"] = tplvars.Webgal(self, self.dir).path()[:-1]

        if not self.dir.source_dir.is_album_root():
            # Parent index link not for album root
            values["parent_index_link"] = self._get_related_index_fn()

        values["osize_index_links"] = self._get_osize_links(self._get_paginated_name())
        values["onum_index_links"] = self._get_onum_links()

        if self.dir.flatten_below():
            values["subgal_links"] = []
        else:
            values["subgal_links"] = self._get_subgal_links()

        values["medias"] = []
        for subdir, medias in self.galleries:
            info = tplvars.Webgal(self, subdir).info()
            if self.size_name in subdir.browse_sizes:
                media_links = [
                    tplvars.media_vars(self, media).full() for media in medias
                ]
            else:
                # This happens when this dir index size is not available in the
                # subdir.
                media_links = []
            values["medias"].append(
                (
                    info,
                    media_links,
                )
            )

        values.update(tplvars.Webgal(self, self.dir).info())

        values["rel_root"] = pathutils.url_path(self.dir.source_dir.rel_root()) + "/"
        values["rel_path"] = pathutils.url_path(self.dir.source_dir.strip_root())

        if self.dir.feed is not None:
            values["feed_url"] = os.path.relpath(self.dir.feed.path, self.dir.path)
            values["feed_url"] = pathutils.url_path(values["feed_url"])
            values["feed_url"] = pathutils.url_quote(values["feed_url"])
        else:
            values["feed_url"] = None

        self.page_template.dump(values, self._path)


class WebalbumFeed(make.FileMakeObject):

    def __init__(self, album, dir_path, pub_url):
        self.path = os.path.join(dir_path, "index.xml")
        super().__init__(self.path)

        self.album = album
        self.pub_url = pub_url
        if not self.pub_url:
            self.pub_url = "http://example.com"
        if not self.pub_url.endswith("/"):
            self.pub_url = self.pub_url + "/"

        self.feed = feeds.RSS20(self.pub_url)
        self.item_template = self.album.theme.tpl_loader.load("feeditem.thtml")

        self.webgals = []

    def set_title(self, title):
        self.feed.title = title

    def set_description(self, description):
        self.feed.description = description

    def push_dir(self, webalbumdir):
        self.webgals.append(webalbumdir)

    def __add_item(self, webalbumdir):
        url = os.path.join(self.pub_url, webalbumdir.source_dir.strip_root())

        desc_values = {}
        desc_values["album_pic_path"] = os.path.join(
            url, webalbumdir.get_webalbumpic_filename()
        )
        desc_values["subgal_count"] = webalbumdir.get_subgal_count()
        desc_values["picture_count"] = webalbumdir.pindex.get_media_count("image")
        desc_values["desc"] = webalbumdir.source_dir.desc
        desc = self.item_template.instanciate(desc_values)

        self.feed.push_item(
            webalbumdir.source_dir.title, url, desc, webalbumdir.source_dir.get_mtime()
        )

    def populate_deps(self):
        wouldbe_deps = sorted(self.webgals, key=lambda s: s.get_mtime())[-10:]

        for webalbumdir in wouldbe_deps:
            if webalbumdir.has_media():
                self.add_dependency(webalbumdir.source_dir)
                self.__add_item(webalbumdir)

    def build(self):
        logging.info(_("FEED %s"), os.path.basename(self.path))
        logging.debug("(%s)", self.path)
        self.feed.dump(self.path)


class SharedFileTemplate(make.FileMakeObject):

    def __init__(self, album, shared_tpl_name, shared_file_dest_tplname, tpl_vars):
        self.album = album
        self.tpl = self.album.theme.tpl_loader.load(shared_tpl_name)
        self.tpl_vars = tpl_vars

        # Remove the 't' from the beginning of ext
        filename, ext = os.path.splitext(shared_file_dest_tplname)
        if ext.startswith(".t"):
            self.path = filename + "." + ext[2:]
        else:
            raise ValueError(
                _(
                    "We have a template with an extension that does not start with a t. Aborting."
                )
            )

        super().__init__(self.path)
        self.add_file_dependency(shared_tpl_name)

    def build(self):
        logging.info(_("TPL %%SHAREDDIR%%/%s"), os.path.basename(self.path))
        logging.debug("(%s)", self.path)
        self.tpl.dump(self.tpl_vars, self.path)


# vim: ts=4 sw=4 expandtab



# Lazygal, a lazy static web gallery generator.
# Copyright (C) 2007-2012 Alexandre Rossi <alexandre.rossi@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


import types
import re
import math


class NewsizeStringParseError(Exception):
    pass


class _Newsize(object):

    def __init__(self, resize_string):
        self.resize_string = resize_string

    def matches(self):
        match = re.match(self.regexp, self.resize_string)
        if match:
            return match
        else:
            raise NewsizeStringParseError

    def dest_size(self, orig_size):
        raise NotImplementedError


class Scale(_Newsize):

    regexp = "^(?P<scale>\\d+)%$"

    def dest_size(self, orig_size):
        match = self.matches()
        scale = int(match.group("scale"))
        return tuple(map(lambda x: x * scale // 100, orig_size))


class XYScale(_Newsize):

    regexp = "^(?P<xscale>\\d+)%(?P<yscale>\\d+)%$"

    def dest_size(self, orig_size):
        match = self.matches()
        xscale = int(match.group("xscale"))
        yscale = int(match.group("yscale"))
        x, y = orig_size
        return (x * xscale // 100, y * yscale // 100)


class Width(_Newsize):

    regexp = "^(?P<width>\\d+)$"

    def dest_size(self, orig_size):
        match = self.matches()
        width = int(match.group("width"))
        x, y = orig_size
        height = y * width // x
        return (width, height)


class Height(_Newsize):

    regexp = "^x(?P<height>\\d+)$"

    def dest_size(self, orig_size):
        match = self.matches()
        height = int(match.group("height"))
        x, y = orig_size
        width = x * height // y
        return (width, height)


class _WidthHeight(_Newsize):

    def requested_widthheight(self):
        match = self.matches()
        width = int(match.group("width"))
        height = int(match.group("height"))
        return width, height

    def appropriate_widthheight(self, orig_size, width, height, constraint):
        x, y = orig_size
        new_height = y * width // x
        if constraint(new_height, height):
            return (width, new_height)
        else:
            new_width = x * height // y
            # y * width / x >= height,
            # therefore x * height / y = new_width <= width
            # with contraint being '<='.
            assert constraint(new_width, width)
            return (new_width, height)


class MaximumWidthHeight(_WidthHeight):

    regexp = "^(?P<width>\\d+)x(?P<height>\\d+)$"

    def dest_size(self, orig_size):
        width, height = self.requested_widthheight()
        return self.appropriate_widthheight(
            orig_size, width, height, lambda x, y: x <= y
        )


class MinimumWidthHeight(_WidthHeight):

    regexp = "^(?P<width>\\d+)x(?P<height>\\d+)\\^$"

    def dest_size(self, orig_size):
        width, height = self.requested_widthheight()
        return self.appropriate_widthheight(
            orig_size, width, height, lambda x, y: x >= y
        )


class MandatoryWidthHeight(_WidthHeight):

    regexp = "^(?P<width>\\d+)x(?P<height>\\d+)!$"

    def dest_size(self, orig_size):
        width, height = self.requested_widthheight()
        return (width, height)


class WidthHeightIfLarger(_WidthHeight):

    regexp = "^(?P<width>\\d+)x(?P<height>\\d+)\\>$"

    def dest_size(self, orig_size):
        width, height = self.requested_widthheight()
        x, y = orig_size
        if x > width or y > height:
            return self.appropriate_widthheight(
                orig_size, width, height, lambda x, y: x <= y
            )
        else:
            return orig_size


class WidthHeightIfSmaller(_WidthHeight):

    regexp = "^(?P<width>\\d+)x(?P<height>\\d+)\\<$"

    def dest_size(self, orig_size):
        width, height = self.requested_widthheight()
        x, y = orig_size
        if x < width and y < height:
            return self.appropriate_widthheight(
                orig_size, width, height, lambda x, y: x >= y
            )
        else:
            return orig_size


class Area(_Newsize):

    regexp = "^(?P<area>\\d+)@$"

    def dest_size(self, orig_size):
        match = self.matches()
        area = int(match.group("area"))
        x, y = orig_size
        # { x0 * y0 = area
        # { x / x0 = y / y0
        # x * y0 / area = y / y0
        # y0 = sqrt( y * area / x )
        # x0 = sqrt( x * area / y )
        return (int(math.sqrt(x * area // y)), int(math.sqrt(y * area // x)))


resize_patterns = []
for name, obj in list(globals().items()):
    if not name.startswith("_") and isinstance(obj, type) and issubclass(obj, _Newsize):
        resize_patterns.append(obj)


def get_newsizer(resize_string):
    for newsizer_class in resize_patterns:
        newsizer = newsizer_class(resize_string)
        try:
            newsizer.matches()
        except NewsizeStringParseError:
            # This is not the syntax used
            pass
        else:
            return newsizer
    raise NewsizeStringParseError


def is_known_newsizer(resize_string):
    for newsizer_class in resize_patterns:
        try:
            newsizer_class(resize_string).matches()
        except NewsizeStringParseError:
            # This is not the syntax used
            pass
        else:
            return True
    raise NewsizeStringParseError


# vim: ts=4 sw=4 expandtab



# Lazygal, a lazy static web gallery generator.
# Copyright (C) 2007-2012 Alexandre Rossi <alexandre.rossi@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


import random
from PIL import Image, ImageChops, ImageFilter


class Color:
    TRANSPARENT = (0, 0, 0, 0)


class PictureMess:

    STEP = 5
    THUMB_HOW_MANY = 5
    THUMB_MAX_ROTATE_ANGLE = 40
    THUMB_WHITE_WIDTH = 5

    def __init__(
        self,
        images_paths,
        top_image_path=None,
        bg="transparent",
        result_size=(
            200,
            150,
        ),
    ):
        if len(images_paths) > self.THUMB_HOW_MANY:
            self.images_paths = random.sample(images_paths, self.THUMB_HOW_MANY)
        else:
            self.images_paths = images_paths

        if top_image_path:
            # Put the top image as the last in the list (the top of the image
            # stack).
            if top_image_path in self.images_paths:
                self.images_paths.remove(top_image_path)
            elif len(self.images_paths) >= self.THUMB_HOW_MANY:
                self.images_paths.pop()
            self.images_paths.append(top_image_path)

        self.bg = bg != "transparent" and bg or Color.TRANSPARENT

        self.result_size = result_size
        self.thumb_size = [3 * max(self.result_size) // 5 for i in range(2)]

        self.picture_mess = None

    def __build_mess_thumb(self, image_path):
        white = maxi = None
        with open(image_path, "rb") as img_fp:
            img = Image.open(img_fp)
            img.thumbnail(self.thumb_size, Image.LANCZOS)
            white_size = [x + 2 * self.THUMB_WHITE_WIDTH for x in img.size]
            white = Image.new("RGB", white_size, "white")
            white.paste(img, (self.THUMB_WHITE_WIDTH, self.THUMB_WHITE_WIDTH))

        maxi = 2 * max(white_size)

        thumb = Image.new("RGBA", (maxi, maxi))

        thumb.paste(white, ((maxi - white_size[0]) // 2, (maxi - white_size[1]) // 2))

        rotation = random.randint(
            -self.THUMB_MAX_ROTATE_ANGLE, self.THUMB_MAX_ROTATE_ANGLE
        )
        thumb = thumb.rotate(rotation, resample=Image.BILINEAR)

        thumb = thumb.crop(thumb.getbbox())
        thumb.thumbnail(self.thumb_size, Image.LANCZOS)
        return thumb

    def __rand_coord_with_step(self, coord, holding_coord):
        return random.randint(0 + self.STEP, holding_coord - coord - self.STEP)

    def __place_thumb_box(self, thumb):
        x_to_fit = self.__rand_coord_with_step(thumb.size[0], self.picture_mess.size[0])
        y_to_fit = self.__rand_coord_with_step(thumb.size[1], self.picture_mess.size[1])
        return (x_to_fit, y_to_fit)

    def __paste_img_to_mess_top(self, img, pos):
        # http://www.mail-archive.com/image-sig@python.org/msg03387.html
        img_without_alpha = img.convert("RGB")

        if self.picture_mess.mode == "RGBA":
            invert_alpha = ImageChops.invert(
                Image.merge("L", self.picture_mess.split()[3:])
            )
            if invert_alpha.size != img.size:
                w, h = img.size
                box = pos + (pos[0] + w, pos[1] + h)
                invert_alpha = invert_alpha.crop(box)
        else:
            invert_alpha = None

        self.picture_mess.paste(img_without_alpha, pos, img)

        if invert_alpha:
            dest_alpha = Image.merge("L", self.picture_mess.split()[3:])
            self.picture_mess.paste(img_without_alpha, pos, invert_alpha)
            self.picture_mess.putalpha(dest_alpha)

    def __add_img_to_mess_top(self, img):
        self.__paste_img_to_mess_top(img, self.__place_thumb_box(img))

    def __build_picture_mess(self):
        self.picture_mess = Image.new("RGBA", self.result_size)
        added_one_thumb = False
        for image_path in self.images_paths:
            try:
                mess_thumb = self.__build_mess_thumb(image_path)
            except IOError:
                # Do not add this thumb to the picture mess
                pass
            else:
                self.__add_img_to_mess_top(mess_thumb)
                added_one_thumb = True
        if not added_one_thumb:
            raise ValueError("No readable image found in submitted list.")

    def write(self, output_file):
        self.__build_picture_mess()

        mode = self.bg == Color.TRANSPARENT and "RGBA" or "RGB"
        shadow = Image.new(mode, self.picture_mess.size, self.bg)

        shadow.paste("black", None, self.picture_mess)
        shadow = shadow.filter(ImageFilter.BLUR)

        tmp = self.picture_mess
        self.picture_mess = shadow

        self.__paste_img_to_mess_top(tmp, (0, 0))
        self.picture_mess.save(output_file)


class PictureTidy(PictureMess):

    THUMB_MAX_ROTATE_ANGLE = 0


WEBALBUMPIC_TYPES = {
    "messy": PictureMess,
    "tidy": PictureTidy,
}


if __name__ == "__main__":
    from optparse import OptionParser

    parser = OptionParser()
    parser.add_option("-s", "--seed", type="int", dest="seed")
    parser.add_option("-t", "--type", dest="type", action="store_true", default=False)
    parser.add_option(
        "-b", "--background", type="string", dest="color", default="transparent"
    )
    (options, args) = parser.parse_args()

    if options.seed:
        random.seed(options.seed)

    MultipicRepr = PictureMess
    if options.type:
        MultipicRepr = WEBALBUMPIC_TYPES[options.type]
    MultipicRepr(args[0:], bg=options.color).write("test.png")


# vim: ts=4 sw=4 expandtab



# Lazygal, a lazy static web gallery generator.
# Copyright (C) 2019 Alexandre Rossi <alexandre.rossi@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

import os
import glob
import logging
import json
import codecs
import locale

from . import pathutils


DEFAULT_THEME = "nojs"
USER_THEME_DIR = os.path.expanduser(os.path.join("~", ".lazygal", "themes"))
THEME_SHARED_FILE_PREFIX = "SHARED_"
THEME_MANIFEST = "manifest.json"


class Theme(object):

    def __init__(self, themes_dir, name):
        self.name = name
        self.themes_dir = themes_dir

        # First try user directory
        self.tpl_dir = os.path.join(USER_THEME_DIR, self.name)
        if not os.path.exists(self.tpl_dir):
            # Fallback to system themes
            self.tpl_dir = os.path.join(themes_dir, self.name)
            if not os.path.exists(self.tpl_dir):
                raise ValueError(_("Theme %s not found") % self.name)
        self.tpl_dir = os.path.abspath(self.tpl_dir)

        self.__load_manifest()

    def prepare_tpl_loader(self, tplfactory):
        self.tpl_loader = tplfactory(
            os.path.join(self.themes_dir, DEFAULT_THEME), self.tpl_dir
        )

        # Load styles templates
        for style in self.get_avail_styles():
            style_filename = style["filename"]
            try:
                self.tpl_loader.load(style_filename)
            except ValueError:
                # Not a known emplate ext, ignore
                pass

        # find out theme kind
        try:
            self.tpl_loader.load("dynindex.thtml")
        except:
            self.kind = "static"
        else:
            self.kind = "dynamic"

    def __load_manifest(self):
        theme_manifest_path = os.path.join(self.tpl_dir, THEME_MANIFEST)
        theme_manifest = {}
        try:
            with codecs.open(
                theme_manifest_path, "r", locale.getpreferredencoding()
            ) as f:
                theme_manifest.update(json.load(f))
        except IOError:
            logging.debug(_("Theme %s does not have a %s"), self.name, THEME_MANIFEST)
        except ValueError:
            logging.error(_("Theme %s : %s parsing error"), self.name, THEME_MANIFEST)
            raise

        if "shared" not in theme_manifest:
            theme_manifest["shared"] = []

        self.shared_files = []

        for s in theme_manifest["shared"]:
            shared = s.copy()
            if type(s["path"]) is not list:
                s["path"] = [s["path"]]

            for path in s["path"]:
                if not path.startswith("/"):
                    path = os.path.join(self.tpl_dir, path)

                if os.path.isfile(path):
                    shared["source"] = os.path.abspath(path)

            if "dest" in s:
                shared["dest"] = s["dest"]
                if shared["dest"].endswith("/"):  # this is a directory
                    shared["dest"] = os.path.join(
                        shared["dest"], os.path.basename(shared["source"])
                    )
            else:
                shared["dest"] = os.path.basename(s["source"])

            shared["abs_dest"] = os.path.join(self.tpl_dir, shared["dest"])

            self.shared_files.append(shared)

        for s in glob.glob(os.path.join(self.tpl_dir, THEME_SHARED_FILE_PREFIX + "*")):
            dest_fn = os.path.basename(s)[len(THEME_SHARED_FILE_PREFIX) :]
            self.shared_files.append(
                {
                    "source": s,
                    "path": [s],
                    "dest": dest_fn,
                    "abs_dest": os.path.join(self.tpl_dir, dest_fn),
                }
            )

    def get_missing_external_assets(self):
        return [s for s in self.shared_files if "source" not in s]

    def check_shared_files(self):
        missing = self.get_missing_external_assets()
        if missing:
            raise ValueError(
                _(
                    "Theme %s: %s reference error: "
                    "files not found:\n%s"
                    "\n\nThe following actions may fix this:\n"
                    "\t- Running ./setup.py dl_assets\n"
                    "\t- Installing deps from your distribution"
                )
                % (
                    self.name,
                    THEME_MANIFEST,
                    "\n".join(["\t" + m["abs_dest"] for m in missing]),
                )
            )

    def get_avail_styles(self, default_style=None):
        style_files_mask = os.path.join(
            self.tpl_dir, THEME_SHARED_FILE_PREFIX + "*" + "css"
        )
        styles = []
        found_default = default_style is None
        for style_tpl_file in glob.glob(style_files_mask):
            style = {}
            tpl_filename = os.path.basename(style_tpl_file).split(".")[0]
            style["filename"] = tpl_filename[len(THEME_SHARED_FILE_PREFIX) :]
            style["name"] = style["filename"].replace("_", " ")
            if default_style is not None:
                if style["filename"] == default_style:
                    style["rel"] = "stylesheet"
                    found_default = True
                else:
                    style["rel"] = "alternate stylesheet"
            styles.append(style)

        if not found_default:
            logging.error(_("Unknown default style '%s'") % default_style)

        return styles


def get_themes():
    source_theme_dir = os.path.join(os.path.dirname(__file__), "..", "themes")
    return [Theme(source_theme_dir, tn) for tn in os.listdir(source_theme_dir)]


# vim: ts=4 sw=4 expandtab



# Lazygal, a lazy static web gallery generator.
# Copyright (C) 2007-2012 Alexandre Rossi <alexandre.rossi@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

from __future__ import division

import os
import locale
import logging
import codecs
import datetime

import gi

gi.require_version("GExiv2", "0.10")
from gi.repository import GExiv2, GLib

from . import make
from . import mediautils

from fractions import Fraction

FILE_METADATA_ENCODING = locale.getpreferredencoding()


MATEW_TAGS = {
    "album_name": "Album name",
    "album_description": "Album description",
    "album_picture": "Album image identifier",
}
MATEW_METADATA = "album_description"

FILE_METADATA = (
    "album-name",
    "album-description",
    "album-picture",
)
FILE_METADATA_MEDIA_SUFFIX = ".comment"

FALLBACK_ENCODING = "utf-8"  # encoding used when guessing

# As per http://www.exiv2.org/tags.html
VENDOR_EXIF_CODES = (
    "Exif.Canon.LensModel",
    "Exif.Minolta.LensID",
    "Exif.Nikon3.Lens",
    "Exif.Nikon3.LensType",
    "Exif.OlympusEq.LensModel",
    "Exif.OlympusEq.LensType",
    "Exif.Panasonic.LensType",
    "Exif.Pentax.LensType",
    "Exif.Samsung2.LensType",
    "Exif.Sigma.LensRange",
    "Exif.Sony1.LensID",
)

GEXIV2_DATE_FORMAT = "%Y:%m:%d %H:%M:%S"
GExiv2.log_set_level(GExiv2.LogLevel.MUTE)  # hide exiv2 errors


TAGS_OF_INTEREST = (
    "Exif.Photo.DateTimeOriginal",
    "Exif.Photo.DateTimeDigitized",
    "Exif.Image.DateTime",
)


def decode_exif_user_comment(raw, imgpath):
    """
    GExiv2 does not decode EXIF user comment.
    """
    # This field can contain charset information
    if raw.startswith("charset="):
        tokens = raw.split(" ")
        csetfield = tokens[0]
        text = " ".join(tokens[1:])
        ignore, cset = csetfield.split("=")
        cset = cset.strip('"')
    else:
        cset = None
        text = raw

    return text


def parse_date(s):
    d = None
    last_e = None
    for f in (
        "%Y-%m-%dT%H:%M:%S.%f%z",
        "%Y-%m-%dT%H:%M:%S%z",
        "%Y-%m-%dT%H:%M:%S.%f",
        "%Y-%m-%dT%H:%M:%S",
    ):
        try:
            d = datetime.datetime.strptime(s, f)
        except ValueError as e:
            last_e = e  # try next format
        else:
            break  # parsing successful

    if d:
        return d
    else:
        raise last_e


class FileMetadata(object):

    def __init__(self, path):
        self.path = path

    def contents(self, splitter=None, firstline=False):
        try:
            with codecs.open(self.path, "r", FILE_METADATA_ENCODING) as f:
                if firstline:
                    c = f.readline()
                else:
                    c = f.read()
        except IOError:
            return None

        if splitter is None or (firstline and splitter == "\n"):
            return c.strip()
        else:
            return map(lambda s: s.strip(), c.split(splitter))


class ImageInfoTags(object):

    def __init__(self, image_path):
        self.image_path = image_path
        try:
            self._metadata = GExiv2.Metadata(self.image_path)
        except GLib.Error as e:
            raise ValueError("cannot load metadata: %s" % e)

    def get_date(self):
        """
        Get real time when photo has been taken. We prefer EXIF fields
        as those were filled by camera, Image DateTime can be updated by
        software when editing photos later.
        """
        for tag in (
            "Exif.Photo.DateTimeOriginal",
            "Exif.Photo.DateTimeDigitized",
        ):
            try:
                dt_str = self._metadata[tag]
                dt = datetime.datetime.strptime(dt_str, GEXIV2_DATE_FORMAT)
            except (KeyError, ValueError) as StrptimeError:
                # ValueError: bypass errors such as "time data '0000:00:00
                # 00:00:00' does not match format '%Y:%m:%d %H:%M:%S'"
                pass
            else:
                return dt

    def get_required_rotation(self):
        try:
            orientation_code = int(self._metadata["Exif.Image.Orientation"])
            if orientation_code == 8:
                return 90
            elif orientation_code == 3:
                return 180
            elif orientation_code == 6:
                return 270
            else:  # Should be orientation_code == 1 but catch all
                return 0
        except KeyError:
            return 0

    def get_camera_name(self):
        """
        Gets vendor and model name from EXIF and tries to construct
        camera name out of this. This is a bit fuzzy, because diferent
        vendors put different information to both tags.
        """
        try:
            model = self._metadata["Exif.Image.Model"].strip()
            # Terminate string at \x00
            pos = model.find("\x00")
            if pos != -1:
                model = model[:14]
            try:
                vendor = self._metadata["Exif.Image.Make"].strip()
                vendor_l = vendor.lower()
                model_l = model.lower()
                # Split vendor to words and check whether they are
                # already in model, for example:
                # Canon/Canon A40
                # PENTAX Corporation/PENTAX K10D
                # Eastman Kodak Company/KODAK DIGITAL SCIENCE DC260 (V01.00)
                for word in vendor_l.split(" "):
                    if model_l.find(word) != -1:
                        return model
                return " ".join([vendor, model])
            except KeyError:
                return model
        except KeyError:
            return ""

    def get_lens_name(self):
        """
        Return name of used lenses. This usually makes sense only for
        SLR cameras and uses various maker notes.
        """
        vendor_values = []
        for key in VENDOR_EXIF_CODES:
            try:
                v = self._metadata.try_get_tag_interpreted_string(key)
                if v is None:
                    raise KeyError
            except (KeyError, UnicodeDecodeError):
                pass
            else:
                vendor_values.append(v.strip())
        return " ".join([s for s in vendor_values if s])

    def get_exif_string(self, name):
        """
        Reads string from EXIF information.
        """
        return self._metadata[name].strip(" ")

    def get_exif_float(self, name):
        """
        Reads float number from EXIF information (where it is stored as
        fraction).
        """
        val = self.get_exif_float_value(name)
        return str(round(val, 1))

    def get_exif_float_value(self, name):
        """
        Reads float number from EXIF information (where it is stored as
        fraction or int).
        """
        val = self._metadata.get_exif_tag_rational(name)
        return val.numerator / val.denominator

    def _fallback_to_encoding(self, encoded_string, encoding=FALLBACK_ENCODING):
        if encoded_string is None:
            raise ValueError
        if type(encoded_string) is str:
            return encoded_string
        try:
            return encoded_string.decode(encoding)
        except UnicodeDecodeError:
            return encoded_string.decode(encoding, "replace")

    def _tag_fallback_to_encoding(self, tag, encoding=FALLBACK_ENCODING):
        try:
            v = self._metadata[tag]
        except UnicodeDecodeError:
            try:
                v = self._metadata.get_raw(tag)[:-1]  # remove null byte
            except AttributeError:  # GExiv2 < 0.10.3
                logging.warning(_("Encoding for '%s' is bad, ignoring"), tag)
                v = ""
        return self._fallback_to_encoding(v, encoding)

    def get_exif_usercomment(self):
        ret = self._metadata["Exif.Photo.UserComment"].strip(" \0\x00\n")
        ret = decode_exif_user_comment(ret, self.image_path)
        # exiv2 may spit out some useless strings sometimes
        if ret in ("User comments", "binary comment"):
            return ""
        return ret

    def get_file_comment(self):
        fmd = FileMetadata(self.image_path + FILE_METADATA_MEDIA_SUFFIX)
        return fmd.contents()

    def get_comment(self):
        try:
            ret = self.get_file_comment()
            if ret is None:
                ret = self.get_exif_usercomment()
                if ret == "":
                    raise ValueError
        except (ValueError, KeyError):
            try:
                ret = self._metadata["Exif.Image.ImageDescription"]
                ret = self._fallback_to_encoding(ret)
            except (ValueError, KeyError):
                try:
                    ret = self._metadata["Iptc.Application2.ObjectName"]
                    ret = self._fallback_to_encoding(ret)
                except (ValueError, KeyError):
                    ret = self.get_jpeg_comment()
        return ret

    def get_flash(self):
        try:
            flash_info = self._metadata.try_get_tag_interpreted_string(
                "Exif.Photo.Flash"
            )
            return self._fallback_to_encoding(flash_info)
        except (ValueError, KeyError):
            return ""

    def get_exposure(self):
        try:
            return str(self._metadata.get_exif_tag_rational("Exif.Photo.ExposureTime"))
        except (ValueError, KeyError):
            return ""

    def get_iso(self):
        try:
            return self._metadata["Exif.Photo.ISOSpeedRatings"]
        except KeyError:
            return ""

    def get_fnumber(self):
        try:
            val = float(self._metadata.get_exif_tag_rational("Exif.Photo.FNumber"))
        except (KeyError, TypeError):
            return ""
        else:
            return "f/{}".format(val)

    def get_focal_length(self):
        try:
            flen = self._metadata.get_exif_tag_rational("Exif.Photo.FocalLength")
            if not flen:
                raise KeyError
        except KeyError:
            return ""
        else:
            flen = "%s mm" % flen

        try:
            flen35 = self._metadata.get_exif_tag_rational(
                "Exif.Photo.FocalLengthIn35mmFilm"
            )
        except KeyError:
            pass
        else:
            if flen35 != flen:
                flen += _(" (35 mm equivalent: %s mm)") % flen35
            return flen

        try:
            try:
                iwidth = self.get_exif_float_value("Exif.Photo.ImageWidth")
            except (IndexError, KeyError):
                iwidth = self.get_exif_float_value("Exif.Photo.PixelXDimension")

            fresunit = self._metadata["Exif.Photo.FocalPlaneResolutionUnit"]
            factors = {"1": 25.4, "2": 25.4, "3": 10, "4": 1, "5": 0.001}
            try:
                fresfactor = factors[fresunit]
            except IndexError:
                fresfactor = 0

            fxres = self.get_exif_float_value("Exif.Photo.FocalPlaneXResolution")
            try:
                ccdwidth = float(iwidth * fresfactor / fxres)
            except ZeroDivisionError:
                return ""

            foclength = self.get_exif_float_value("Exif.Photo.FocalLength")
            try:
                lenstr = "%.01f" % (foclength / ccdwidth * 36 + 0.5)
            except ZeroDivisionError:
                raise ValueError

            if lenstr != flen:
                flen += _(" (35 mm equivalent: %s mm)") % lenstr
        except (IndexError, KeyError, ValueError):
            return flen

        return flen

    def get_jpeg_comment(self):
        try:
            comment = self._metadata.try_get_comment()
            if comment is None or "\x00" in comment:
                raise ValueError  # ignore missing or broken JPEG comments
            return self._fallback_to_encoding(comment.strip(" "))
        except ValueError:
            return ""

    def get_authorship(self):
        try:
            return self._tag_fallback_to_encoding("Exif.Image.Artist")
        except KeyError:
            return ""

    def get_keywords(self):
        """
        Returns all the image tags in a list.

        Try to find the maximum number of keywords. Photo applications store
        keywords in various places. For a comprehensive list, see
        http://redmine.yorba.org/projects/shotwell/wiki/PhotoTags
        """
        kw = list()
        for key in (
            "Iptc.Application2.Keywords",
            "Xmp.MicrosoftPhoto.LastKeywordXMP",
            "Xmp.dc.subject",
            "Xmp.digiKam.TagsList",
        ):
            try:
                values = self._metadata.try_get_tag_multiple(key)
            except KeyError:
                pass
            except UnicodeDecodeError:
                values = self._metadata.get_raw(key).split(b"\x1c")
            else:
                for value in values:
                    if value:
                        kw.append(self._fallback_to_encoding(value))
        # FIXME
        # Reading the metadata Xmp.lr.hierarchicalSubject produces error
        # messages:
        #   "No namespace info available for XMP prefix `lr'"
        # kw += self._metadata.try_get_tag_multiple('Xmp.lr.hierarchicalSubject')

        # remove duplicates
        kw = set(kw)

        return kw

    def get_location(self):
        loc = self._metadata.try_get_gps_info()
        if loc == (0, 0, 0):
            return None
        else:
            return {
                # https://xkcd.com/2170/
                "latitude": round(loc.latitude, 4),
                "latitudeRef": self._metadata["Exif.GPSInfo.GPSLatitudeRef"],
                "longitude": round(loc.longitude, 4),
                "longitudeRef": self._metadata["Exif.GPSInfo.GPSLongitudeRef"],
                "altitude": round(loc.altitude),
            }

    def of_interest(self):
        return {
            tag: self._metadata[tag]
            for tag in TAGS_OF_INTEREST
            if tag in self._metadata
        }


class VideoInfoTags(object):

    def __init__(self, path):
        videoinfo = mediautils.VideoInfo(path).inspect()
        self.__parse(videoinfo)

    def __parse(self, videoinfo):
        # size
        for s in videoinfo["streams"]:
            if s["codec_type"] == "video":
                self.size = (s["width"], s["height"])
                break
        if self.size is None:  # no video stream
            self.size = (None, None)

        # date
        try:
            raw = videoinfo["format"]["tags"]["creation_time"]
            self.creation_time = parse_date(raw)
        except (KeyError, ValueError):
            self.creation_time = None

    def get_date(self):
        return self.creation_time


class NoMetadata(Exception):
    """
    Exception indicating that no meta data has been found.
    """

    pass


class DirectoryMetadata(make.GroupTask):

    def __init__(self, dir_path):
        super().__init__()

        self.dir_path = dir_path
        self.add_file_dependency(self.dir_path)

        self.description_filename = os.path.join(self.dir_path, MATEW_METADATA)
        if os.path.isfile(self.description_filename):
            self.description_file = self.description_filename
            self.add_file_dependency(self.description_filename)
        else:
            self.description_file = None

            # Add dependency to "file metadata" files if they exist.
            for file_md_fn in FILE_METADATA:
                file_md_path = os.path.join(self.dir_path, file_md_fn)
                if os.path.isfile(file_md_path):
                    self.add_file_dependency(file_md_path)

    def get_matew_metadata(self, metadata, subdir=None):
        """
        Return dictionary with meta data parsed from Matew like format.
        """
        if subdir is None:
            path = self.description_file
        else:
            path = os.path.join(self.dir_path, subdir, MATEW_METADATA)

        if path is None or not os.path.exists(path):
            raise NoMetadata(_("Could not open metadata file %s") % path)

        with open(path, "rb") as f:
            for line in f:
                line = line.decode(FILE_METADATA_ENCODING)
                for tag in MATEW_TAGS.keys():
                    tag_text = MATEW_TAGS[tag]
                    tag_len = len(tag_text)
                    if line[:tag_len] == tag_text:
                        data = line[tag_len:]
                        data = data.strip()
                        # Strip quotes
                        if data[0] == '"':
                            data = data[1:]
                        if data[-1] == '"':
                            data = data[:-1]

                        if tag == "album_picture":
                            if subdir is not None:
                                data = os.path.join(subdir, data)

                        metadata[tag] = data
                        break

        return metadata

    def get_file_metadata(self, metadata, subdir=None):
        """
        Returns the file metadata that could be found in the directory.
        """

        if subdir is None:
            subdir = self.dir_path

        if "album_name" not in metadata.keys():
            fmd = FileMetadata(os.path.join(subdir, "album-name")).contents()
            if fmd is not None:
                metadata["album_name"] = fmd

        if "album_description" not in metadata.keys():
            fmd = FileMetadata(os.path.join(subdir, "album-description")).contents()
            if fmd is not None:
                metadata["album_description"] = fmd

        if "album_picture" not in metadata.keys():
            fmd = FileMetadata(os.path.join(subdir, "album-picture")).contents(
                splitter="\n", firstline=True
            )
            if fmd is not None:
                metadata["album_picture"] = fmd

        return metadata

    def get(self, subdir=None, dir=None):
        """
        Returns directory meta data. First tries to parse known formats
        and then fall backs to built in defaults.
        """

        result = {}

        try:
            result = self.get_matew_metadata(result, subdir)
        except NoMetadata:
            pass

        result = self.get_file_metadata(result, subdir)

        # Add album picture
        if "album_picture" not in result:
            try:
                if dir is not None:
                    picture = dir.get_all_medias_paths()[0]
                else:
                    raise IndexError
            except IndexError:
                picture = None
            if picture is not None:
                result["album_picture"] = os.path.relpath(picture, dir.path)

        return result

    def get_title(self):
        try:
            return self.get()["album_name"]
        except KeyError:
            return os.path.basename(self.dir_path).replace("_", " ")


class DefaultMetadata(make.FileMakeObject):
    """
    This is a the building of the default metadata file in the source directory.
    """

    def __init__(self, source_dir, album):
        self.source_dir = source_dir

        metadata_path = os.path.join(self.source_dir.path, MATEW_METADATA)
        super().__init__(metadata_path)

        self.album = album

    def build(self):
        md = DirectoryMetadata(self.source_dir.path)

        md_data = md.get(None, self.source_dir)
        if "album_description" in md_data.keys() or "album_name" in md_data.keys():
            logging.debug(_("  SKIPPED because metadata exists."))
        elif self.source_dir.get_all_medias_count() < 1:
            logging.debug(_("  SKIPPED because directory does not contain images."))
        else:
            self.generate(md_data)

    def generate(self, md):
        """
        Generates new metadata file with default values.
        """

        logging.info(_("GEN %s"), self._path)

        with codecs.open(self._path, "w", FILE_METADATA_ENCODING) as f:
            f.write("# Directory metadata for lazygal, Matew format\n")
            f.write('Album name "%s"\n' % self.source_dir.human_name)
            f.write('Album description ""\n')
            f.write('Album image identifier "%s"\n' % md["album_picture"])


# vim: ts=4 sw=4 expandtab



# Lazygal, a lazy static web gallery generator.
# Copyright (C) 2007-2012 Alexandre Rossi <alexandre.rossi@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


import fnmatch
import locale
import logging
import datetime
import os
import re
import time

from PIL import Image as PILImage

from . import pathutils, make, metadata
from . import mediautils


SOURCEDIR_CONFIGFILE = ".lazygal"


class File(make.FileSimpleDependency):

    def __init__(self, path, album):
        super().__init__(path)

        self.path = path
        self.album = album
        self.filename = os.path.basename(self.path)
        self.name, self.extension = os.path.splitext(self.filename)

    def strip_root(self, path=None):
        if not path:
            path = self.path

        relative_path = os.path.relpath(path, self.album.source_dir)
        if relative_path == ".":
            return ""
        else:
            return relative_path

    def rel_root(self):
        return os.path.relpath(self.album.source_dir, self.path)

    def rel_path(self, from_dir, path=None):
        try:
            from_dir = from_dir.path
        except AttributeError:
            pass

        if path is None:
            path = self.path

        return os.path.relpath(path, from_dir)

    def is_subdir_of(self, dir, path=None):
        if path is None:
            path = self.path

        try:
            dir_path = dir.path
        except AttributeError:
            dir_path = dir

        return pathutils.is_subdir_of(dir_path, path)

    def get_album_level(self, path=None):
        if path is None:
            path = self.path

        if os.path.isdir(path):
            cur_path = path
        else:
            cur_path = os.path.dirname(path)

        album_level = 0
        while cur_path != self.album.source_dir:
            cur_path, tail = os.path.split(cur_path)
            album_level += 1
            if pathutils.is_root(cur_path):
                raise RuntimeError(_("Root not found"))
        return album_level

    def should_be_skipped(self):
        head = self.strip_root()
        while head != "":
            head, tail = os.path.split(head)
            for pattern in self.album.excludes:
                if fnmatch.fnmatch(tail, pattern):
                    return True
        return False

    def get_datetime(self):
        return datetime.datetime.fromtimestamp(self.get_mtime())

    def name_numeric(self):
        numeric_part = re.sub(r"\D", "", self.filename)
        return numeric_part and int(numeric_part) or 0


class MediaFile(File):

    def __init__(self, path, album):
        super().__init__(path, album)
        self.broken = False
        self.md = {
            "type": self.type,
            "date": None,
            "metadata": {
                "comment": None,
            },
        }
        self.__md_loaded = False

        comment_file_path = self.path + metadata.FILE_METADATA_MEDIA_SUFFIX
        if os.path.isfile(comment_file_path):
            self.comment_file_path = comment_file_path
        else:
            self.comment_file_path = None

    def load_metadata_from_mediafile(self):
        try:
            mdloader = self.mdloader(self.path)
        except ValueError as e:
            logging.debug("Cannot load metadata for %s: %s" % (self.path, e))
            mdloader = None
        self._parse_metadata(mdloader)

    def load_metadata(self, pindex):
        if not self.__md_loaded:
            if not pindex:
                self.load_metadata_from_mediafile()
            elif (
                self.get_mtime() < pindex.get_mtime()
                and self.filename in pindex.data["medias"]
            ):
                # load metadata from persistent index
                self.md = pindex.data["medias"][self.filename]
            else:
                # load metadata from file
                self.load_metadata_from_mediafile()
                pindex.load_media(self)

            assert self.md["date"].__class__ == datetime.datetime
            self.__md_loaded = True

    def _parse_metadata(self, mdloader):
        md_date = None
        if mdloader:
            md_date = mdloader.get_date()
            if md_date:
                self.md["metadata"]["date"] = md_date
                self.md["date"] = md_date

        if not md_date:
            # No date available in md, or bad format, use file mtime
            self.md["date"] = self.get_datetime()

    def set_broken(self):
        self.broken = True

    def has_reliable_date(self):
        return "date" in self.md["metadata"] and self.md["metadata"]["date"]

    def get_date_taken(self):
        return self.md["date"]

    def get_size(self):
        try:
            if self.broken:
                raise KeyError()
            return (self.md["width"], self.md["height"])
        except KeyError:
            self.broken = True
            return (0, 0)

    def sortkey(self):
        # Comparison between 'no md' and 'md' sorts md after
        # (reliable here means encoded by the camera).
        if self.has_reliable_date():
            return (1, self.get_date_taken().timestamp())
        else:
            return (0, self.filename)


class ImageFile(MediaFile):
    type = "image"
    mdloader = metadata.ImageInfoTags

    def probe(self):
        with open(self.path, "rb") as im_fp:
            try:
                im = PILImage.open(im_fp)
            except IOError:
                self.set_broken()
                return (None, None)
            else:
                return {
                    "width": im.size[0],
                    "height": im.size[1],
                    "alphachannel": im.mode in ("RGBA", "LA"),
                }

    def _parse_metadata(self, mdloader):
        super()._parse_metadata(mdloader)

        i = self.probe()
        if self.broken:
            return

        self.md.update(i)

        if mdloader:
            self.md["metadata"].update(
                {
                    "comment": mdloader.get_comment() or None,
                    "rotation": mdloader.get_required_rotation(),
                    "camera_name": mdloader.get_camera_name(),
                    "lens_name": mdloader.get_lens_name(),
                    "flash": mdloader.get_flash(),
                    "exposure": mdloader.get_exposure(),
                    "iso": mdloader.get_iso(),
                    "fnumber": mdloader.get_fnumber(),
                    "focal_length": mdloader.get_focal_length(),
                    "authorship": mdloader.get_authorship(),
                    "keywords": list(mdloader.get_keywords()),
                    "location": mdloader.get_location(),
                }
            )


class VideoFile(MediaFile):
    type = "video"
    mdloader = metadata.VideoInfoTags

    def _parse_metadata(self, mdloader):
        super()._parse_metadata(mdloader)

        if mdloader:
            self.md.update(
                {
                    "width": mdloader.size[0],
                    "height": mdloader.size[1],
                }
            )
        else:
            self.set_broken()


class MediaHandler(object):

    FORMATS = {
        ".jpeg": ImageFile,
        ".jpg": ImageFile,
        ".png": ImageFile,
        ".mov": VideoFile,
        ".avi": VideoFile,
        ".mp4": VideoFile,
        ".3gp": VideoFile,
        ".webm": VideoFile,
        ".mpeg": VideoFile,
        ".gif": VideoFile,
        ".mpg": VideoFile,
        ".mts": VideoFile,
    }

    def __init__(self, album):
        self.album = album

    @staticmethod
    def is_known_media(path, album):
        tail = os.path.basename(path)
        for pattern in album.excludes:
            if fnmatch.fnmatch(tail, pattern):
                return False
        filename, extension = os.path.splitext(path)
        extension = extension.lower()
        return extension in MediaHandler.FORMATS.keys()

    NO_VIDEO_SUPPORT_WARNING_ISSUED = False

    @staticmethod
    def warn_no_video_support():
        if not MediaHandler.NO_VIDEO_SUPPORT_WARNING_ISSUED:
            logging.warning(_("Video support is disabled: could not find ffmpeg"))

    def get_media(self, path):
        tail = os.path.basename(path)
        for pattern in self.album.excludes:
            if fnmatch.fnmatch(tail, pattern):
                return None
        filename, extension = os.path.splitext(path)
        extension = extension.lower()
        if extension in MediaHandler.FORMATS.keys():
            media_class = MediaHandler.FORMATS[extension]
            if media_class == VideoFile and not mediautils.HAVE_VIDEO:
                MediaHandler.warn_no_video_support()
                return None
            return media_class(path, self.album)
        else:
            return None


class Directory(File):

    def __init__(self, source, subdirs, filenames, album):
        super().__init__(source, album)

        # No breaking up of filename and extension for directories
        self.name = self.filename
        self.extension = None

        self.subdirs = subdirs
        self.filenames = filenames

        self.human_name = self.album._str_humanize(self.name)

        media_handler = MediaHandler(self.album)
        self.medias = []
        self.medias_names = []
        for filename in self.filenames:
            media_path = os.path.join(self.path, filename)

            if not os.path.isfile(media_path):
                logging.info(
                    _("  Ignoring %s, cannot open file (broken symlink?)."), filename
                )
                logging.debug("(%s)", os.path.join(self.path, filename))
                continue

            media = media_handler.get_media(media_path)
            if media:
                if media.broken:
                    logging.error(_("  %s is BROKEN, skipped") % media.filename)
                    continue

                self.medias_names.append(filename)
                self.medias.append(media)
            elif not self.is_metadata(filename) and filename != SOURCEDIR_CONFIGFILE:
                logging.info(_("  Ignoring %s, format not supported."), filename)
                logging.debug("(%s)", os.path.join(self.path, filename))

        self.metadata = metadata.DirectoryMetadata(self.path)
        md = self.metadata.get(None, self)
        if "album_name" in md.keys():
            self.title = md["album_name"]
        else:
            self.title = self.human_name
        if "album_description" in md.keys():
            self.desc = md["album_description"]
        else:
            self.desc = None

        if "album_picture" in md.keys():
            self.album_picture = md["album_picture"]
        else:
            self.album_picture = None

    def is_album_root(self):
        return self.path == self.album.source_dir

    def parent_paths(self):
        parent_paths = [self.path]

        found = False
        head = self.path

        while not found:
            if head == self.album.source_dir:
                found = True
            elif pathutils.is_root(head):
                raise RuntimeError(_("Root not found"))
            else:
                head, tail = os.path.split(head)
                parent_paths.append(os.path.join(self.path, head))

        return parent_paths

    def parents_metadata(self):
        return map(metadata.DirectoryMetadata, self.parent_paths())

    def is_metadata(self, filename):
        if filename == metadata.MATEW_METADATA:
            return True
        if filename in metadata.FILE_METADATA:
            return True

        # Check for media metadata
        related_media = filename[: -len(metadata.FILE_METADATA_MEDIA_SUFFIX)]
        # As the list self.medias_names is being constructed while this
        # check takes place, the following is only reliable if the filenames
        # list is sorted (thus t.jpg.comment is after t.jpg, and t.jpg is
        # already in self.medias_names), which is the case.
        if related_media in self.medias_names:
            return True

        return False

    def get_media_count(self, media_type=None):
        if media_type is None:
            return len(self.medias_names)
        else:
            typed_media_count = 0
            for media in self.medias:
                if media.type == media_type:
                    typed_media_count += 1
            return typed_media_count

    def get_all_medias_count(self, media_type=None):
        all_medias_count = self.get_media_count(media_type)
        for subdir in self.subdirs:
            all_medias_count += subdir.get_all_medias_count(media_type)
        return all_medias_count

    def get_all_medias(self):
        all_medias = list(self.medias)  # We want a copy here.
        for subdir in self.subdirs:
            all_medias.extend(subdir.get_all_medias())
        return all_medias

    def get_all_medias_paths(self):
        return [m.path for m in self.get_all_medias()]

    def get_all_subdirs(self):
        all_subdirs = list(self.subdirs)  # We want a copy here.
        for subdir in self.subdirs:
            all_subdirs.extend(subdir.get_all_subdirs())
        return all_subdirs

    def latest_media_stamp(self, from_media=False):
        """
        Returns the latest media date:
            - first considering all pics that have a MD date
            - if none have a reliable date, use file mtimes.
        """
        all_medias = self.get_all_medias()
        media_stamp_max = None
        for m in all_medias:
            m.load_metadata(not from_media)
            if m.has_reliable_date():
                media_stamp = m.get_date_taken().timestamp()
                if media_stamp_max is None or media_stamp > media_stamp_max:
                    media_stamp_max = media_stamp

        if media_stamp_max is None:
            # none of the media had a reliable date, use mtime instead
            for m in all_medias:
                media_stamp = m.get_datetime().timestamp()
                if media_stamp_max is None or media_stamp > media_stamp_max:
                    media_stamp_max = media_stamp

        return media_stamp_max


# vim: ts=4 sw=4 expandtab



# Lazygal, a lazy static web gallery generator.
# Copyright (C) 2010-2020 Alexandre Rossi <alexandre.rossi@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


import json
import logging
import math
import os
import shutil
import subprocess
import tempfile


from PIL import Image as PILImage


class VideoError(Exception):
    pass


FFMPEG = shutil.which("ffmpeg")
FFPROBE = shutil.which("ffprobe")
HAVE_VIDEO = FFMPEG and FFPROBE


class VideoProcessor(object):

    def __init__(self, input_file):
        self.progress = None

        self.global_opts = [
            "-nostdin",
            "-progress",
            "-",
            "-y",  # force overwrite existing file
        ]
        self.input_file_opts = ["-hwaccel", "auto"]
        self.input_file = input_file
        self.output_file_opts = []

        self.videofilters = []
        self.duration = None

    def set_progress(self, progress):
        self.progress = progress

    def scale(self, newsize):
        self.videofilters.append("scale=%d:%d" % (newsize[0], newsize[1]))

    def parse_time(self, time_str):
        h, m, s = time_str.split(":")
        return 3600 * int(h) + 60 * int(m) + float(s)

    def parse_output(self, line):
        if line.startswith("Duration:"):
            self.duration = self.parse_time(line.split(" ")[1].strip(","))
        elif self.duration and line.startswith("out_time="):
            try:
                position = self.parse_time(line.split("=")[1])
            except ValueError:
                position = 0
            percent = math.floor(100 * position / self.duration)
            if self.progress:
                self.progress.set_task_progress(percent)
            else:
                logging.info("progress: %d%%" % percent)

    def convert(self, outfile):
        runcmd = [FFMPEG]
        runcmd.extend(self.global_opts)
        runcmd.extend(self.input_file_opts)
        runcmd.extend(["-i", self.input_file])
        if self.videofilters:
            runcmd.extend(["-vf", ",".join(self.videofilters)])
        runcmd.extend(self.output_file_opts)
        runcmd.append(outfile)
        logging.debug("RUNNING %s" % " ".join(runcmd))
        with subprocess.Popen(
            runcmd,
            text=True,
            errors="replace",
            stdout=subprocess.PIPE,
            stderr=subprocess.STDOUT,
        ) as p:
            for line in p.stdout:
                line = line.strip()
                logging.debug(line)
                self.parse_output(line)
        if p.returncode != 0:
            raise VideoError("%s failed" % " ".join(runcmd))


class VideoInfo(object):

    def __init__(self, path):
        self.path = path

    def inspect(self):
        try:
            info = subprocess.check_output(
                [
                    FFPROBE,
                    "-v",
                    "error",
                    "-print_format",
                    "json",
                    "-show_format",
                    "-show_streams",
                    self.path,
                ],
                stderr=subprocess.DEVNULL,
            )
        except subprocess.CalledProcessError:
            raise ValueError("cannot load metadata")
        return json.loads(info)


class VideoTranscoder(VideoProcessor):

    def __init__(self, mediapath, videocodec, audiocodec):
        super().__init__(mediapath)
        self.output_file_opts.extend(["-c:v", videocodec, "-c:a", audiocodec])
        self.videofilters.append("format=yuv420p")


class WebMTranscoder(VideoTranscoder):

    def __init__(self, mediapath):
        super().__init__(mediapath, "libvpx-vp9", "libopus")
        self.output_file_opts.extend(
            [
                "-crf",
                "22",
                "-b:v",
                "2000k",
            ]
        )


class MP4Transcoder(VideoTranscoder):

    def __init__(self, mediapath):
        super().__init__(mediapath, "libx264", "aac")
        self.output_file_opts.extend(
            ["-movflags", "faststart", "-preset", "slow", "-crf", "22"]
        )


class VideoFramesExtractor(VideoProcessor):

    def __init__(self, input_file, resize=None, scene=True, frames=10):
        super().__init__(input_file)

        if scene:
            self.videofilters.append("select=gt(scene\\,0.4)")
            self.videofilters.append("fps=1/60")
        else:  # in case of very short video with one scene
            self.videofilters.append("fps=10")

        if resize is not None:
            self.scale(resize)

        self.output_file_opts.extend(["-frames:v", str(frames), "-vsync", "vfr"])

    def convert(self, outfile):
        super(VideoFramesExtractor, self).convert(outfile)
        if not os.path.isfile(outfile % 1):
            # some ffmpeg version do not extract any frame but do not exit
            # with return code
            raise VideoError("no frame extracted")


class VideoThumbnailer(object):

    def __init__(self, video):
        self.video = video

    def find_most_representative(self, images):
        images = list(images)
        logging.debug("find_most_reprensentative in %s" % " ".join(images))
        if not images:
            return None
        if len(images) == 1:
            return images[0]

        histograms = []
        for path in images:
            with open(path, "rb") as im_fp:
                im = PILImage.open(im_fp)
                histograms.append((path, im.histogram()))

        n_samples = len(histograms)
        n_values = len(histograms[0][1])

        # Average each histogram value
        average_hist = []
        for value_index in range(n_values):
            average = 0.0
            for path, histogram in histograms:
                average = average + (float(histogram[value_index]) / n_samples)
            average_hist.append(average)

        # Find histogram closest to average histogram
        min_mse = None
        best_frame_no = 0
        for hist_index in range(len(histograms)):
            hist = histograms[hist_index][1]
            mse = 0.0
            for value_index in range(n_values):
                gap = average_hist[value_index] - hist[value_index]
                mse = mse + gap * gap

            if min_mse is None or mse < min_mse:
                min_mse = mse
                best_frame_no = hist_index

        return histograms[best_frame_no][0]

    def convert(self, outfile, resize=None):
        tmpdir = tempfile.mkdtemp(prefix="lazygal-")

        try:
            tmpimg_name = os.path.join(
                tmpdir, "%s_%%s_%%%%d.jpg" % os.path.basename(self.video)
            )

            try:
                fextractor = VideoFramesExtractor(self.video, resize)
                fextractor.convert(tmpimg_name % "scene")
            except VideoError as e:
                fextractor = VideoFramesExtractor(self.video, resize, scene=False)
                fextractor.convert(tmpimg_name % "noscene")

            best = self.find_most_representative(
                map(lambda fn: os.path.join(tmpdir, fn), os.listdir(tmpdir))
            )
            if not best:
                raise VideoError("no best frame found")
            shutil.copyfile(best, outfile)
        except VideoError:
            raise
        finally:
            shutil.rmtree(tmpdir)


if __name__ == "__main__":
    import sys
    import os

    logger = logging.getLogger()
    logger.setLevel(logging.DEBUG)

    converter_types = sys.argv[1].split(",")
    converters = {}
    for converter_type in converter_types:
        if converter_type == "webm":
            converter = WebMTranscoder
        elif converter_type == "mp4":
            converter = MP4Transcoder
        elif converter_type == "jpeg":
            converter = VideoThumbnailer
        else:
            raise ValueError("unknwon converter type %s" % converter_type)
        converters[converter_type] = converter

    for file_path in sys.argv[2:]:
        videoinfo = VideoInfo(file_path)
        i = videoinfo.inspect()
        for s in i["streams"]:
            if s["codec_type"] == "video":
                print(
                    "Video is %dx%d, %ds"
                    % (s["width"], s["height"], round(float(s["duration"])))
                )
                break  # search only first video stream

        fn, ext = os.path.splitext(os.path.basename(file_path))
        for converter_type, converter in converters.items():

            counter_str = ""
            counter = 0
            filename_free = False
            while not filename_free:
                target_path = fn + counter_str + "." + converter_type
                if os.path.isfile(target_path):
                    counter = counter + 1
                    counter_str = "_%d" % counter
                else:
                    filename_free = True

            converter(file_path).convert(target_path)


# vim: ts=4 sw=4 expandtab



#!/usr/bin/env python3
# -*- coding: UTF-8 -*-
# Lazygal, a lazy static web gallery generator.
# Copyright (C) 2007-2012 Michal Čihař, Mickaël Royer, Alexandre Rossi
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


import glob
import gettext
import json
import locale
import os
import os.path
import re
import shutil
import stat
import sys
import urllib.request


import setuptools
import setuptools.command.build_py


import lazygal


gettext.install("lazygal")


def newer(fpath1, fpath2):
    if os.path.isfile(fpath2):
        return os.path.getmtime(fpath1) > os.path.getmtime(fpath2)
    else:
        return True


class test_lazygal(setuptools.Command):

    description = "Run the test suite"
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        import lazygaltest

        lazygaltest.run()


class dl_assets(setuptools.Command):

    description = "Download extra assets if not provided by the system"
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        import lazygal.theme

        for t in lazygal.theme.get_themes():
            for a in t.get_missing_external_assets():
                asset_file = urllib.request.urlopen(a["url"])
                setuptools.log.info(
                    "downloading %s into %s" % (a["url"], a["abs_dest"])
                )
                with open(a["abs_dest"], "wb") as output:
                    output.write(asset_file.read())


class sample_album(setuptools.Command):

    description = "Generate a sample album for testing purposes"
    user_options = [("outdir=", None, "Output directory")]

    def initialize_options(self):
        self.outdir = None

    def finalize_options(self):
        assert self.outdir is not None, "Output directory is mandatory"

    def run(self):
        import lazygaltest

        lazygaltest.sample_album(self.outdir)


class build_manpages(setuptools.Command):

    description = "Build manpages"
    user_options = []

    manpages = None
    mandir = os.path.join(os.path.dirname(__file__), "man")
    executable = shutil.which("pandoc")

    def initialize_options(self):
        pass

    def finalize_options(self):
        self.manpages = glob.glob(os.path.join(self.mandir, "*.md"))

    def __get_man_section(self, filename):
        # filename should be file.mansection.md
        return filename.split(".")[-2]

    def run(self):
        data_files = self.distribution.data_files

        for manpagesrc in self.manpages:
            manpage = os.path.splitext(manpagesrc)[0]  # remove '.md' at the end
            section = manpage[-1:]
            if newer(manpagesrc, manpage):
                cmd = [self.executable, "-s", "-t", "man", "-o", manpage, manpagesrc]
                self.spawn(cmd)

            targetpath = os.path.join("share", "man", "man%s" % section)
            data_files.append(
                (
                    targetpath,
                    (manpage,),
                )
            )


class build_i18n_lazygal(setuptools.Command):

    description = "Build i18n files"
    user_options = []
    po_package = None
    po_directory = None
    po_files = None

    def initialize_options(self):
        pass

    def finalize_options(self):
        self.po_directory = "locale"
        self.po_package = "lazygal"
        self.po_files = glob.glob(os.path.join(self.po_directory, "*.po"))

    def run(self):
        data_files = self.distribution.data_files

        for po_file in self.po_files:
            lang = os.path.basename(po_file[:-3])
            mo_dir = os.path.join("build", "mo", lang, "LC_MESSAGES")
            mo_file = os.path.join(mo_dir, "%s.mo" % self.po_package)
            if not os.path.exists(mo_dir):
                os.makedirs(mo_dir)

            cmd = ["msgfmt", po_file, "-o", mo_file]
            self.spawn(cmd)

            targetpath = os.path.join("share/locale", lang, "LC_MESSAGES")
            data_files.append((targetpath, (mo_file,)))


class build_lazygal(setuptools.command.build_py.build_py):

    def __has_manpages(self, command):
        return (
            "build_manpages" in self.distribution.cmdclass
            and build_manpages.executable is not None
        )

    def __has_i18n(self, command):
        return "build_i18n" in self.distribution.cmdclass

    def run(self):
        if build_manpages.executable is not None:
            self.run_command("build_manpages")
        self.run_command("build_i18n")
        super(build_lazygal, self).run()


# list themes to install
theme_data = []
themes = glob.glob(os.path.join("themes", "*"))
for theme in themes:
    themename = os.path.basename(theme)
    theme_data.append(
        (
            os.path.join("share", "lazygal", "themes", themename),
            glob.glob(os.path.join("themes", themename, "*")),
        )
    )


setuptools.setup(
    name="lazygal",
    version=lazygal.__version__,
    description="Static web gallery generator",
    long_description="",
    author="Alexandre Rossi",
    author_email="alexandre.rossi@gmail.com",
    maintainer="Alexandre Rossi",
    maintainer_email="alexandre.rossi@gmail.com",
    platforms=["Linux", "Mac OSX", "Windows XP/2000/NT", "Windows 95/98/ME"],
    keywords=["gallery", "exif", "photo", "image"],
    url="https://sml.zincube.net/~niol/repositories.git/lazygal/about/",
    download_url="https://sml.zincube.net/~niol/repositories.git/lazygal/",
    license="GPL-2.0-or-later",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Win32 (MS Windows)",
        "Environment :: X11 Applications :: GTK",
        "Intended Audience :: End Users/Desktop",
        "Operating System :: Microsoft :: Windows :: Windows 95/98/2000",
        "Operating System :: Microsoft :: Windows :: Windows NT/2000",
        "Operating System :: POSIX",
        "Operating System :: Unix",
        "Programming Language :: Python",
        "Topic :: Utilities",
        "Natural Language :: English",
    ],
    packages=["lazygal"],
    package_data={
        "lazygal": ["defaults.json"],
    },
    entry_points={
        "console_scripts": [
            "lazygal = lazygal.cmdline:main",
        ]
    },
    cmdclass={
        "build_py": build_lazygal,
        "build_i18n": build_i18n_lazygal,
        "build_manpages": build_manpages,
        "dl_assets": dl_assets,
        "test": test_lazygal,
        "sample_album": sample_album,
    },
    data_files=theme_data,
)

# vim: ts=4 sw=4 expandtab



# coding=utf-8
#
# Lazygal, a lazy static web gallery generator.
# Copyright (C) 2011-2012 Alexandre Rossi <alexandre.rossi@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


import unittest
import os
import posixpath
import ntpath
from . import LazygalTest
from lazygal.pathutils import *
from lazygal.sourcetree import Directory


class TestPathutils(LazygalTest):

    def setUp(self):
        super().setUp()
        self.test_root = self.get_working_path()

    def d(self, dpath):
        """
        Create a working directory in a test location if it does not exist.
        """
        if dpath[0] == "/":
            dpath = dpath[1:]

        dpath = os.path.join(self.test_root, dpath)
        if not os.path.exists(dpath):
            os.makedirs(dpath)
        assert os.path.isdir(dpath)
        return dpath

    def f(self, fpath):
        """
        Create a working file in a test location if it does not exist.
        """
        dpath = self.d(os.path.dirname(fpath))
        fpath = os.path.join(dpath, os.path.basename(fpath))
        if not os.path.exists(fpath):
            with open(fpath, "a"):
                os.utime(fpath, None)
        assert os.path.isfile(fpath)
        return fpath

    def test_is_subdir_of_dirs(self):
        self.assertTrue(is_subdir_of(self.d("/tmp"), self.d("/tmp/foo")))
        self.assertFalse(is_subdir_of(self.d("/tmp"), self.d("/tmpx/foo")))
        self.assertTrue(is_subdir_of(self.d("/tmp/bar"), self.f("/tmp/bar/baz/jay")))
        self.assertFalse(is_subdir_of(self.d("/tmp/john/mail"), self.d("/tmpz")))

    def test_is_subdir_of_files(self):
        self.assertTrue(is_subdir_of(self.d("/tmp"), self.f("/tmp/foo")))
        self.assertFalse(is_subdir_of(self.d("/tmp"), self.f("/tmpx/foo")))
        self.assertTrue(is_subdir_of(self.d("/tmp/bar"), self.f("/tmp/bar/baz/jay")))
        self.assertFalse(is_subdir_of(self.d("/tmp/john/mail"), self.f("/tmpz")))

    def test_url_path(self):
        self.assertEqual(url_path("/usr/bin/lazygal", posixpath), "/usr/bin/lazygal")
        self.assertEqual(url_path("../bin/lazygal", posixpath), "../bin/lazygal")
        self.assertEqual(
            url_path("C:\\Program Files\\Lazygal\\Lazygal.exe", ntpath),
            "Program Files/Lazygal/Lazygal.exe",
        )
        self.assertEqual(
            url_path("..\\Lazygal\\Lazygal.exe", ntpath), "../Lazygal/Lazygal.exe"
        )

    def test_url_quote(self):
        self.assertEqual(
            url_quote("http://host:8080/~user/album/"), "http://host:8080/~user/album/"
        )
        self.assertEqual(
            url_quote("https://host/~user/album/"), "https://host/~user/album/"
        )
        self.assertEqual(url_quote("/albumé/"), "/album%C3%A9/")
        self.assertEqual(url_quote("filter: user/"), "filter%3A%20user/")
        self.assertEqual(url_quote("index.html", "id"), "index.html#id")


if __name__ == "__main__":
    unittest.main()


# vim: ts=4 sw=4 expandtab



# coding=utf-8
# Lazygal, a lazy static web gallery generator.
# Copyright (C) 2010-2012 Alexandre Rossi <alexandre.rossi@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


import unittest
import os
import locale
import datetime
import time
import codecs


from . import LazygalTest
from lazygal import mediautils
from lazygal import metadata

metadata.FILE_METADATA_ENCODING = "utf-8"  # force for these tests
from lazygal.generators import Album
from lazygal.sourcetree import Directory


class TestFileMetadata(LazygalTest):

    def setUp(self):
        super().setUp()

        self.source_dir = self.get_working_path()
        album = Album(self.source_dir)
        self.album_root = Directory(self.source_dir, [], [], album)

    def test_album_name(self):
        album_name = "Album de <b>l'école<b>"
        self.create_file(os.path.join(self.source_dir, "album-name"), album_name)

        md = metadata.DirectoryMetadata(self.album_root.path)
        self.assertEqual(md.get()["album_name"], album_name)

    def test_album_desc(self):
        album_desc = "Allons voir la fête de l'école tous ensemble"
        self.create_file(os.path.join(self.source_dir, "album-description"), album_desc)

        md = metadata.DirectoryMetadata(self.album_root.path)
        self.assertEqual(md.get()["album_description"], album_desc)

    def test_img_desc(self):
        img_path = self.add_img(self.source_dir, "captioned_pic.jpg")

        # Set dumy comment which should be ignored.
        im = metadata.GExiv2.Metadata(img_path)
        im["Exif.Photo.UserComment"] = "comment not to show"
        im.save_file()

        # Output real comment which should be chosen over the dummy comment.
        image_caption = "Les élèves forment une ronde dans la <em>cour</em>."
        self.create_file(
            os.path.join(self.source_dir, img_path + ".comment"), image_caption
        )

        imgmd = metadata.ImageInfoTags(img_path)
        self.assertEqual(imgmd.get_comment(), image_caption)

    def test_album_picture(self):
        img_names = (
            "ontop.jpg",
            "second.jpg",
        )
        for img_name in img_names:
            self.add_img(self.source_dir, img_name)
        self.create_file(
            os.path.join(self.source_dir, "album-picture"), "\n".join(img_names)
        )

        # Reload the directory now that the metadata file is present
        self.album_root = Directory(self.source_dir, [], [], self.album_root.album)

        self.assertEqual(img_names[0], self.album_root.album_picture)

    def test_matew_metadata_album_picture(self):
        img_names = ("ontop.jpg", "second.jpg", "realtitle.jpg")
        for img_name in img_names:
            self.add_img(self.source_dir, img_name)

        self.create_file(
            os.path.join(self.source_dir, "album_description"),
            """
Album image identifier "realtitle.jpg"
""",
        )

        # Reload the directory now that the metadata file is created
        self.album_root = Directory(self.source_dir, [], [], self.album_root.album)

        self.assertEqual(img_names[2], self.album_root.album_picture)

    def test_auto_metadata_album_picture(self):
        img_names = (
            "ontop.jpg",
            "second.jpg",
        )
        for img_name in img_names:
            self.add_img(self.source_dir, img_name)

        self.album_root = Directory(
            self.source_dir, [], img_names, self.album_root.album
        )

        self.assertEqual(img_names[0], self.album_root.album_picture)

    def test_gen_metadata(self):
        img_names = (
            "first\xe3.jpg",
            "second.jpg",
        )
        for img_name in img_names:
            self.add_img(self.source_dir, img_name)

        self.album_root.album.generate_default_metadata()

        with codecs.open(
            os.path.join(self.source_dir, "album_description"), "r", "utf-8"
        ) as f:
            fmd = f.read()

        self.assertEqual(
            fmd,
            """# Directory metadata for lazygal, Matew format
Album name "%s"
Album description ""
Album image identifier "first\xe3.jpg"
"""
            % os.path.basename(self.source_dir).replace("_", " "),
        )

    def test_comment_none(self):
        im_md = metadata.ImageInfoTags(self.get_sample_path("sample.jpg"))
        self.assertEqual(im_md.get_comment(), "")

    def test_image_description(self):
        sample = "sample-image-description.jpg"
        im_md = metadata.ImageInfoTags(self.get_sample_path(sample))

        self.assertEqual(im_md.get_comment(), "test ImageDescription")

    def test_jpeg_comment(self):
        sample = "sample-jpeg-comment.jpg"
        im_md = metadata.ImageInfoTags(self.get_sample_path(sample))

        self.assertEqual(im_md.get_comment(), "test jpeg comment")

    def test_jpeg_comment_unicode(self):
        sample = "sample-jpeg-comment-unicode.jpg"
        im_md = metadata.ImageInfoTags(self.get_sample_path(sample))

        self.assertEqual(im_md.get_comment(), "test jpeg comment éù")

    def test_usercomment_ascii(self):
        sample = "sample-usercomment-ascii.jpg"
        im_md = metadata.ImageInfoTags(self.get_sample_path(sample))

        self.assertEqual(im_md.get_comment(), "deja vu")

    def test_usercomment_unicode_le(self):
        sample = "sample-usercomment-unicode-ii.jpg"
        im_md = metadata.ImageInfoTags(self.get_sample_path(sample))

        self.assertEqual(im_md.get_comment(), "unicode test éà")

    def test_usercomment_unicode_be(self):
        sample = "sample-usercomment-unicode-mm.jpg"
        im_md = metadata.ImageInfoTags(self.get_sample_path(sample))

        self.assertEqual(im_md.get_comment(), "unicode test : éàê")

    def test_usercomment_empty_and_encoding(self):
        """
        Exif.Photo.UserComment: charset="Ascii"

        Exif.Image.ImageDescription: unset
        Iptc.Application2.ObjectName: Jasper the Bear

        In this example, the IPTC tag should be chosen because the
        EXIF photo user comment should be considered empty.
        """
        sample = "sample-usercomment-empty-and-encoding.jpg"
        im_md = metadata.ImageInfoTags(self.get_sample_path(sample))

        self.assertEqual(im_md.get_comment(), "Jasper the Bear")

    def test_exif_date(self):
        sample = "sample.jpg"
        im_md = metadata.ImageInfoTags(self.get_sample_path(sample))
        d = im_md.get_date()
        self.assertEqual(d, datetime.datetime(2010, 2, 5, 23, 56, 24))

        sample = "sample-model-nikon1.jpg"
        im_md = metadata.ImageInfoTags(self.get_sample_path(sample))
        d = im_md.get_date()
        self.assertEqual(d, datetime.datetime(2011, 5, 21, 22, 19, 30))

    def test_model(self):
        sample = "sample-model-nikon1.jpg"
        im_md = metadata.ImageInfoTags(self.get_sample_path(sample))
        self.assertEqual(im_md.get_camera_name(), "NIKON D5000")

    def test_lens(self):
        sample = "sample-model-nikon1.jpg"
        im_md = metadata.ImageInfoTags(self.get_sample_path(sample))
        self.assertEqual(im_md.get_lens_name(), "35mm F1.8 D G")

        sample = "sample-model-nikon2.jpg"
        im_md = metadata.ImageInfoTags(self.get_sample_path(sample))
        self.assertEqual(im_md.get_lens_name(), "70-200mm F2.8 D G")

        sample = "sample-model-pentax1.jpg"
        im_md = metadata.ImageInfoTags(self.get_sample_path(sample))
        self.assertEqual(im_md.get_lens_name(), "smc PENTAX-DA 18-55mm F3.5-5.6 AL WR")

        sample = "sample-bad-lens.jpg"
        im_md = metadata.ImageInfoTags(self.get_sample_path(sample))
        self.assertEqual(im_md.get_lens_name(), "")

    def test_flash(self):
        sample = "sample-model-pentax1.jpg"
        im_md = metadata.ImageInfoTags(self.get_sample_path(sample))
        self.assertEqual(im_md.get_flash(), "Yes, auto")

    def test_focal_length(self):
        sample = "sample-model-pentax1.jpg"
        im_md = metadata.ImageInfoTags(self.get_sample_path(sample))
        self.assertEqual(im_md.get_focal_length(), "18 mm (35 mm equivalent: 27 mm)")

    def test_authorship(self):
        sample = "sample-author-badencoding.jpg"
        im_md = metadata.ImageInfoTags(self.get_sample_path(sample))
        self.assertEqual(
            im_md.get_authorship(),
            "\ufffd\ufffd\ufffd\ufffd\ufffd\ufffd\ufffd\ufffd\ufffd \ufffd\ufffd\ufffd\ufffd\ufffd\ufffd\ufffd",
        )

    def test_keywords(self):
        sample = "sample-image-keywords.jpg"
        im_md = metadata.ImageInfoTags(self.get_sample_path(sample))
        self.assertEqual(
            im_md.get_keywords(),
            set(
                [
                    "lazygal",
                    "Iptc.Application2.Keywords.lazygal",
                    "Xmp.MicrosoftPhoto.LastKeywordXMP.lazygal",
                    "Xmp.dc.subject.lazygal",
                    "Xmp.digiKam.TagsList.lazygal",
                    #'Xmp.lr.hierarchicalSubject.lazygal'
                ]
            ),
        )

    def test_invalid_iptc_keyword(self):
        sample = "sample-bad-iptc-keywords.jpg"
        im_md = metadata.ImageInfoTags(self.get_sample_path(sample))
        self.assertEqual(
            im_md._metadata.get_raw("Iptc.Application2.Keywords"),
            b"Anton\x1c\x1c\x1c\x1cBj\xf6rn",
        )
        self.assertEqual(
            im_md.get_keywords(),
            {"Personen/Kollegen/Björn", "Anton", "Personen/Familie/Anton", "Björn"},
        )

    def test_location(self):
        sample = "sample.jpg"
        im_md = metadata.ImageInfoTags(self.get_sample_path(sample))
        self.assertEqual(im_md.get_location(), None)

        sample = "sample-with-gps.jpg"
        im_md = metadata.ImageInfoTags(self.get_sample_path(sample))
        self.assertEqual(
            im_md.get_location(),
            {
                "latitude": 47.0636,
                "latitudeRef": "N",
                "longitude": 8.6893,
                "longitudeRef": "E",
                "altitude": 1527,
            },
        )

    @unittest.skipIf(not mediautils.HAVE_VIDEO, "video support not available")
    def test_video(self):
        sample = "vid.mov"
        vid_md = metadata.VideoInfoTags(self.get_sample_path(sample))
        self.assertEqual(
            vid_md.get_date(),
            datetime.datetime(2007, 2, 10, 17, 51, 8, tzinfo=datetime.timezone.utc),
        )


if __name__ == "__main__":
    unittest.main()


# vim: ts=4 sw=4 expandtab



# Lazygal, a lazy static web gallery generator.
# Copyright (C) 2007-2012 Alexandre Rossi <alexandre.rossi@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


import os
import locale
import logging
import zipfile

from . import make
from . import pathutils


class WebalbumFile(make.FileMakeObject):

    def __init__(self, path, dir):
        super().__init__(path)
        self.dir = dir
        self.path = path

    def rel_path(self, dir, url=False):
        """
        Returns the path of the current object relative to the supplied dir
        object argument. Force forward slashes if url is True.
        """
        ret = None
        if dir is None or dir is self.dir:
            ret = os.path.basename(self.path)
        else:
            ret = dir.rel_path(self.path)

        if url:
            return pathutils.url_path(ret)
        else:
            return ret


class MediaOriginal(WebalbumFile):

    def __init__(self, dir, source_media):
        self.filename = source_media.filename
        path = os.path.join(dir.path, self.filename)
        super().__init__(path, dir)

        self.source_media = source_media

        self.set_dep_only()

    def get_size(self):
        return self.source_media.get_size()


class CopyMediaOriginal(MediaOriginal):

    def __init__(self, dir, source_media):
        super().__init__(dir, source_media)
        self.add_dependency(make.FileCopy(self.source_media.path, self.path))

    def build(self):
        logging.info("  CP %s", self.filename)
        logging.debug("(%s)", self.path)


class SymlinkMediaOriginal(MediaOriginal):

    def __init__(self, dir, source_media):
        super().__init__(dir, source_media)
        self.add_dependency(make.FileSymlink(self.source_media.path, self.path))

    def build(self):
        logging.info("  SYMLINK %s", self.filename)
        logging.debug("(%s)", self.path)


class WebalbumArchive(WebalbumFile):

    def __init__(self, webgal_dir):
        self.filename = webgal_dir.source_dir.name + ".zip"
        self.path = os.path.join(webgal_dir.path, self.filename)
        super().__init__(self.path, webgal_dir)

        self.add_dependency(self.dir.source_dir)
        self.pics = list(
            map(
                lambda x: os.path.join(self.dir.source_dir.path, x.media.filename),
                self.dir.medias,
            )
        )
        for pic in self.pics:
            self.add_file_dependency(pic)

    def build(self):
        zip_rel_path = self.rel_path(self.dir.flattening_dir)
        logging.info(_("  ZIP %s"), zip_rel_path)
        logging.debug("(%s)", self.path)

        with zipfile.ZipFile(self.path, "w") as archive:
            for pic in self.pics:
                inzip_fn = os.path.join(self.dir.source_dir.name, os.path.basename(pic))
                archive.write(pic, inzip_fn)

    def size(self):
        return os.path.getsize(self.path)


class SharedFileCopy(make.FileCopy):

    def __init__(self, src, dst):
        super().__init__(src, dst)

    def build(self):
        logging.info(_("CP %%SHAREDDIR%%/%s"), os.path.basename(self.path))
        logging.debug("(%s)", self.path)
        super().build()


# vim: ts=4 sw=4 expandtab



# Lazygal, a lazy static web gallery generator.
# Copyright (C) 2011-2012 Alexandre Rossi <alexandre.rossi@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


import os
import time
import unittest

from . import LazygalTestGen
from lazygal.generators import WebalbumDir
from lazygal.sourcetree import Directory
from lazygal.genpage import WebalbumIndexPage


class TestDeps(LazygalTestGen):
    """
    Dependencies of built items
    """

    def test_second_build(self):
        """
        Once built, a webgal shall not need build.
        """
        source_subgal = self.setup_subgal(
            "subgal", ["subgal_img.jpg", "subgal_img2.jpg"]
        )

        dest_path = os.path.join(self.tmpdir, "dst")

        webgal = WebalbumDir(source_subgal, [], self.album, dest_path)
        self.assertTrue(
            webgal.needs_build_quick(),
            "Webalbum subgal has not been built and does not need build.",
        )

        self.album.generate(dest_path)

        webgal = WebalbumDir(source_subgal, [], self.album, dest_path)

        self.assertFalse(
            webgal.needs_build(),
            "Webalbum subgal has been built and still needs build.",
        )

        # second pass
        self.album.generate(dest_path)

        # forced second pass
        self.album.config.set("runtime", "check-all-dirs", True)
        self.album.generate(dest_path)

    def test_dirmetadata_update(self):
        """
        Updated directory metadata file shall trigger the rebuild of the
        corresponding webgal directory.
        """
        subgal_path = os.path.join(self.source_dir, "subgal")
        os.mkdir(subgal_path)
        self.add_img(subgal_path, "subgal_img.jpg")

        # metadata must exist before creating the Directory() object (md files
        # are probed in the constructor.
        self.album.generate_default_metadata()

        source_subgal = Directory(subgal_path, [], ["subgal_img.jpg"], self.album)

        dest_path = os.path.join(self.tmpdir, "dst")

        self.album.generate(dest_path)
        dest_subgal = WebalbumDir(source_subgal, [], self.album, dest_path)

        self.assertFalse(
            dest_subgal.needs_build(),
            "Webalbum subgal has been built and still needs build.",
        )

        # touch the description file
        os.utime(os.path.join(source_subgal.path, "album_description"), None)
        # New objects in order to probe filesystem
        source_subgal = Directory(subgal_path, [], ["subgal_img.jpg"], self.album)
        dest_subgal = WebalbumDir(source_subgal, [], self.album, dest_path)

        self.assertTrue(
            dest_subgal.needs_build(),
            "Webalbum subgal should need build because of updated dir md.",
        )

    def test_subgal_update(self):
        """
        Updated subgals shall trigger the webgal rebuild and the parent
        directory index rebuild.
        """
        source_subgal = self.setup_subgal("subgal", ["subgal_img.jpg"])

        dest_path = os.path.join(self.tmpdir, "dst")

        self.album.generate(dest_path)
        dest_subgal = WebalbumDir(source_subgal, [], self.album, dest_path)

        self.assertFalse(
            dest_subgal.needs_build(),
            "Webalbum subgal has been built and still needs build.",
        )

        self.add_img(source_subgal.path, "subgal_img2.jpg")
        # New objects to ensure pic is taken into account
        source_subgal = Directory(
            source_subgal.path, [], ["subgal_img.jpg", "subgal_img2.jpg"], self.album
        )
        dest_subgal = WebalbumDir(source_subgal, [], self.album, dest_path)

        # Subgal should need build.
        self.assertTrue(
            dest_subgal.needs_build(),
            "Webalbum subgal should need build because of added pic in subgal.",
        )

        # Parent directory should need build.
        source_gal = Directory(self.source_dir, [source_subgal], [], self.album)
        dest_gal = WebalbumDir(source_gal, [dest_subgal], self.album, dest_path)
        self.assertTrue(
            dest_gal.needs_build(),
            "Webalbum gal should need build because of added pic in subgal.",
        )

        # Parent directory should need build.
        parent_index = WebalbumIndexPage(
            dest_gal, "small", 0, [dest_subgal], [(dest_subgal, dest_subgal.medias)]
        )
        self.assertTrue(
            parent_index.needs_build(),
            "Webalbum gal index should need build because of added pic in subgal.",
        )


if __name__ == "__main__":
    unittest.main()


# vim: ts=4 sw=4 expandtab



# Lazygal, a lazy static web gallery generator.
# Copyright (C) 2013 Alexandre Rossi <alexandre.rossi@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


import os
import re
import posixpath

from . import pathutils


FILESIZE_UNIT_PREFIXES = (
    ("T", 2**40),
    ("G", 2**30),
    ("M", 2**20),
    ("K", 2**10),
)


def format_filesize(size_bytes):
    for unit_prefix, limit in FILESIZE_UNIT_PREFIXES:
        if size_bytes >= limit:
            return "%.1f %siB" % (round(float(size_bytes) / limit, 1), unit_prefix)
    return "%.1f B" % size_bytes


class TemplateVariables(object):

    def __init__(self, page):
        self.page = page


class Media(TemplateVariables):

    def __init__(self, page, webalbum_media):
        super().__init__(page)
        self.webalbum_media = webalbum_media
        self.media = webalbum_media.media

    def link(self):
        if self.webalbum_media:
            link_vals = {}

            link_vals["type"] = self.media.type

            if self.page.dir.album.theme.kind == "static":
                link_vals["link"] = self.webalbum_media.browse_pages[
                    self.page.size_name
                ].rel_path(self.page.dir, url=True)
            elif self.page.dir.album.theme.kind == "dynamic":
                link_vals["link"] = self.webalbum_media.resized[
                    self.page.size_name
                ].rel_path(self.page.dir, url=True)
            link_vals["link"] = pathutils.url_quote(link_vals["link"])

            link_vals["thumb"] = self.webalbum_media.thumb.rel_path(
                self.page.dir, url=True
            )
            link_vals["thumb"] = pathutils.url_quote(link_vals["thumb"])

            if not self.media.broken:
                link_vals["thumb_width"], link_vals["thumb_height"] = (
                    self.webalbum_media.thumb.get_size()
                )

            link_vals["thumb_name"] = self.page.dir.album._str_humanize(self.media.name)

            if self.page.dir.original:
                if self.page.dir.orig_base:
                    link_vals["original_link"] = posixpath.join(
                        pathutils.url_path(self.page.dir.source_dir.rel_root()),
                        self.page.dir.orig_base,
                        pathutils.url_path(self.page.dir.source_dir.strip_root()),
                        self.media.filename,
                    )
                else:
                    link_vals["original_link"] = self.media.filename
                link_vals["original_link"] = pathutils.url_quote(
                    link_vals["original_link"]
                )

            return link_vals
        else:
            return None


class Image(Media):

    def full(self):
        tpl_values = self.link()
        tpl_values["img_src"] = self.webalbum_media.resized[
            self.page.size_name
        ].filename
        tpl_values["img_src"] = pathutils.url_quote(tpl_values["img_src"])

        tpl_values["image_name"] = self.media.filename

        tpl_values["img_width"], tpl_values["img_height"] = self.webalbum_media.resized[
            self.page.size_name
        ].get_size()

        if self.page.dir.config.get("webgal", "publish-metadata"):
            tpl_values["publish_metadata"] = True
            tpl_values.update(self.media.md["metadata"])

            if tpl_values["comment"]:
                tpl_values["comment"] = self.page._do_not_escape(tpl_values["comment"])

            if "location" in tpl_values and not self.page.dir.config.get(
                "webgal", "keep-gps"
            ):
                del tpl_values["location"]

        return tpl_values


class Video(Media):

    def full(self):
        tpl_values = self.link()
        tpl_values["video_src"] = self.webalbum_media.resized[
            self.page.size_name
        ].rel_path(self.page.dir, url=True)
        tpl_values["video_src"] = pathutils.url_quote(tpl_values["video_src"])
        return tpl_values


def media_vars(page, webalbum_media):
    cls = None
    if webalbum_media.media.type == "image":
        cls = Image
    elif webalbum_media.media.type == "video":
        cls = Video
    else:
        raise NotImplementedError
    return cls(page, webalbum_media)


class SrcPath(TemplateVariables):

    def __init__(self, page, srcpath):
        super().__init__(page)
        self.srcpath = srcpath

    def should_be_flattened(self):
        return self.page.dir.should_be_flattened(self.srcpath)

    def id(self):
        if self.should_be_flattened():
            rawid = self.page.dir.source_dir.rel_path(
                self.page.dir.flattening_srcpath(self.srcpath), self.srcpath
            )
        else:
            rawid = os.path.basename(self.srcpath)
        return re.sub(r"[/ \\]", "_", rawid)

    def link(self):
        link_target = self.page._add_size_qualifier("index.html")

        # Add anchor target to get straight to gallery listing
        anchor = self.id() if self.should_be_flattened() else ""

        # Add relative path to link if needed
        index_path = None
        if self.should_be_flattened():
            index_path = self.page.dir.flattening_srcpath(self.srcpath)
        elif self.srcpath != self.page.dir.source_dir.path:
            index_path = self.srcpath
        if index_path is not None:
            index_path = self.page.dir.rel_path_to_src(index_path)
            index_path = pathutils.url_path(index_path)
            link_target = posixpath.join(index_path, link_target)

        return pathutils.url_quote(link_target, anchor=anchor)

    def path(self):
        wg_path = []
        for dirmd in self.page.dir.source_dir.parents_metadata():
            wg = {}
            wg["link"] = SrcPath(self.page, dirmd.dir_path).link()
            wg["name"] = dirmd.get_title()
            wg["root"] = dirmd.dir_path == self.page.dir.album.source_dir
            wg["current"] = dirmd.dir_path == self.page.dir.source_dir.path
            wg_path.append(wg)

        wg_path.reverse()
        return wg_path


class Webgal(SrcPath):

    def __init__(self, page, webgal):
        super().__init__(page, webgal.source_dir.path)
        self.webgal = webgal

    def info(self):
        dir_info = {}
        if self.webgal.source_dir.metadata:
            dir_info.update(self.webgal.source_dir.metadata.get())

        for html_key in ("album_name", "album_description"):
            if html_key in dir_info:
                dir_info[html_key] = self.page._do_not_escape(dir_info[html_key])

        if "album_name" not in dir_info:
            dir_info["album_name"] = self.webgal.source_dir.human_name

        dirzip = self.webgal.webassets.data["dirzip"]
        if dirzip:
            archive_rel_dir = self.webgal.rel_path(self.page.dir.path)
            archive_rel_path = posixpath.join(archive_rel_dir, dirzip["filename"])
            dir_info["dirzip"] = pathutils.url_quote(archive_rel_path)
            dir_info["dirzip_size"] = dirzip["sizestr"]

        dir_info["is_main"] = self.webgal is self.page.dir

        dir_info["image_count"] = self.webgal.pindex.get_media_count("image")
        dir_info["subgal_count"] = len(self.webgal.source_dir.subdirs)

        dir_info["id"] = pathutils.url_quote(self.id())

        return dir_info

    def link_info(self):
        link_info = self.info()
        link_info["link"] = self.link()
        link_info["album_picture"] = posixpath.join(
            self.webgal.source_dir.name, self.webgal.get_webalbumpic_filename()
        )
        link_info["album_picture"] = pathutils.url_quote(link_info["album_picture"])
        return link_info


# vim: ts=4 sw=4 expandtab
